@@ -9,7 +9,7 @@ demean <- function(x, fl, weights, ..., means = FALSE) {
   }
   msg <- "For higher-dimensional centering and projecting out interactions need to install.packages('%s'), then unload [detach('package:collapse', unload = TRUE)] and reload [library(collapse)]."
   res <- getenvFUN("fixest_demean", msg)(x, fl, attr(fl, "slope.vars"), attr(fl, "slope.flag"),
-                                         weights = weights, ..., im_confident = TRUE)
+                                         weights = weights, ..., notes = FALSE, im_confident = TRUE)
   if(!means) return(duplAttributes(res, x))
     # if(!is.matrix(x)) dim(res) <- NULL # also need for flmres... e.g. with weights... intercept is no longer always added, so res needs to be a matrix...
     # Need matrix dimensions... for subset in variable.wise... do.call(cbind, fl[!fc]) needs to be preserved... # return(if(means) x - drop(res) else drop(res))
@@ -326,11 +326,14 @@ fhdwithin.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.me
     return(setAttributes(x, ax))
   } else return(setAttributes(demean(x, fl, w, ...), ax))
 }
-fhdwithin.pseries <- function(x, effect = seq_col(attr(x, "index")), w = NULL, na.rm = TRUE, fill = TRUE, ...) {
-  ix <- attr(x, "index")
+fhdwithin.pseries <- function(x, effect = "all", w = NULL, na.rm = TRUE, fill = TRUE, ...) {
+  ix <- findex(x)
   namix <- attr(ix, "names")
-  effect <- cols2int(effect, ix, namix)
+  if(is.character(effect) && length(effect) == 1L && effect == "all") {
+    effect <- seq_along(namix)
+  } else effect <- cols2int(effect, ix, namix)
   g <- .subset(ix, effect)
+
   if(na.rm && length(cc <- whichv(x, NA, TRUE)) != length(x)) {
      g <- .Call(C_subsetDT, g, cc, seq_along(g), FALSE) # lapply(g, `[`, cc) -> slower !
     if(fill) {
@@ -408,10 +411,12 @@ fhdwithin.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.met
 }
 
 # x = collapse:::colsubset(pwlddev, is.numeric)
-fhdwithin.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NULL, na.rm = TRUE, fill = TRUE, variable.wise = TRUE, ...) {
-  ix <- attr(x, "index")
+fhdwithin.pdata.frame <- function(x, effect = "all", w = NULL, na.rm = TRUE, fill = TRUE, variable.wise = TRUE, ...) {
+  ix <- findex(x)
   namix <- attr(ix, "names")
-  effect <- cols2int(effect, ix, namix)
+  if(is.character(effect) && length(effect) == 1L && effect == "all") {
+    effect <- seq_along(namix)
+  } else effect <- cols2int(effect, ix, namix)
   g <- .subset(ix, effect)
 
   if(na.rm && fill && variable.wise) {
@@ -439,7 +444,7 @@ fhdwithin.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NUL
       reix <- copyMostAttributes(c(.Call(C_subsetDT, ix, cc, toss, FALSE), gcc)[namix], ix)
     } else reix <- copyMostAttributes(gcc, ix)
     attr(reix, "row.names") <- .set_row_names(length(cc))
-    attr(Y, "index") <- reix
+    attr(Y, if(inherits(x, "indexed_frame")) "index_df" else "index") <- reix
     attr(Y, "na.rm") <- which(miss)
     return(Y)
   } else return(demean(x, g, w, ...)) # setAttributes(, ax) -> Not needed anymore (included in demean())
@@ -515,6 +520,8 @@ fhdwithin.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, va
   }
 }
 
+fhdwithin.list <- function(x, ...) fhdwithin.data.frame(x, ...)
+
 
 # Note: could also do Mudlack and add means to second regression -> better than two-times centering ??
 HDW <- function(x, ...) UseMethod("HDW") # , x
@@ -524,7 +531,7 @@ HDW.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.method =
   fhdwithin.default(x, fl, w, na.rm, fill, lm.method, ...)
 }
 
-HDW.pseries <- function(x, effect = seq_col(attr(x, "index")), w = NULL, na.rm = TRUE, fill = TRUE, ...)
+HDW.pseries <- function(x, effect = "all", w = NULL, na.rm = TRUE, fill = TRUE, ...)
   fhdwithin.pseries(x, effect, w, na.rm, fill, ...)
 
 HDW.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, stub = "HDW.", lm.method = "qr", ...)
@@ -542,7 +549,7 @@ HDW.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
       fl[[2L]] <- NULL
     } else {
       fvars <- ckmatch(all.vars(fl), nam)
-      Xvars <- if(length(cols)) fsetdiff(cols2int(cols, x, nam), fvars) else seq_along(unclass(x))[-fvars]
+      Xvars <- cols2intrmgn(fvars, cols, x) # if(length(cols)) fsetdiff(cols2int(cols, x, nam), fvars) else seq_along(unclass(x))[-fvars]
     }
     ax[["names"]] <- if(is.character(stub)) paste0(stub, nam[Xvars]) else nam[Xvars]
 
@@ -593,10 +600,11 @@ HDW.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
  add_stub(fhdwithin.data.frame(if(is.null(cols)) x else colsubset(x, cols), fl, w, na.rm, fill, variable.wise, lm.method, ...), stub)
 }
 
-HDW.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NULL, cols = is.numeric, na.rm = TRUE, fill = TRUE,
+HDW.pdata.frame <- function(x, effect = "all", w = NULL, cols = is.numeric, na.rm = TRUE, fill = TRUE,
                             variable.wise = TRUE, stub = "HDW.", ...)
-add_stub(fhdwithin.pdata.frame(if(is.null(cols)) x else colsubset(x, cols), effect, w, na.rm, fill, variable.wise, ...), stub)
+add_stub(fhdwithin.pdata.frame(fcolsubset(x, cols2intrmgn(which(attr(x, "names") %in% attr(findex(x), "nam")), cols, x)), effect, w, na.rm, fill, variable.wise, ...), stub)
 
+HDW.list <- function(x, ...) HDW.data.frame(x, ...)
 
 # Theory: y = ?1 x1 + ?2 x2 + e
 # FWT: M2 y = ?1 M2 x1 + e so residuals: e = M2 y - ?1 M2 x1 and fitted:
@@ -674,7 +682,7 @@ fhdbetween.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.m
 }
 
 
-fhdbetween.pseries <- function(x, effect = seq_col(attr(x, "index")), w = NULL, na.rm = TRUE, fill = TRUE, ...)
+fhdbetween.pseries <- function(x, effect = "all", w = NULL, na.rm = TRUE, fill = TRUE, ...)
   fhdwithin.pseries(x, effect, w, na.rm, fill, ..., means = TRUE)
 
 fhdbetween.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.method = "qr", ...) {
@@ -736,7 +744,7 @@ fhdbetween.matrix <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.me
   } else return(setAttributes(demean(x, fl, w, ..., means = TRUE), ax))
 }
 
-fhdbetween.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NULL, na.rm = TRUE, fill = TRUE, variable.wise = TRUE, ...)
+fhdbetween.pdata.frame <- function(x, effect = "all", w = NULL, na.rm = TRUE, fill = TRUE, variable.wise = TRUE, ...)
   fhdwithin.pdata.frame(x, effect, w, na.rm, fill, variable.wise, ..., means = TRUE)
 
 
@@ -813,6 +821,8 @@ fhdbetween.data.frame <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, v
   }
 }
 
+fhdbetween.list <- function(x, ...) fhdbetween.data.frame(x, ...)
+
 
 HDB <- function(x, ...) UseMethod("HDB") # , x
 
@@ -821,7 +831,7 @@ HDB.default <- function(x, fl, w = NULL, na.rm = TRUE, fill = FALSE, lm.method =
   fhdbetween.default(x, fl, w, na.rm, fill, lm.method, ...)
 }
 
-HDB.pseries <- function(x, effect = seq_col(attr(x, "index")), w = NULL, na.rm = TRUE, fill = TRUE, ...)
+HDB.pseries <- function(x, effect = "all", w = NULL, na.rm = TRUE, fill = TRUE, ...)
   fhdwithin.pseries(x, effect, w, na.rm, fill, ..., means = TRUE)
 
 
@@ -839,7 +849,7 @@ HDB.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
       fl[[2L]] <- NULL
     } else {
       fvars <- ckmatch(all.vars(fl), nam)
-      Xvars <- if(length(cols)) fsetdiff(cols2int(cols, x, nam), fvars) else seq_along(unclass(x))[-fvars]
+      Xvars <- cols2intrmgn(fvars, cols, x) #  if(length(cols)) fsetdiff(cols2int(cols, x, nam), fvars) else seq_along(unclass(x))[-fvars]
     }
     ax[["names"]] <- if(is.character(stub)) paste0(stub, nam[Xvars]) else nam[Xvars]
 
@@ -897,10 +907,11 @@ HDB.data.frame <- function(x, fl, w = NULL, cols = is.numeric, na.rm = TRUE, fil
 }
 
 
-HDB.pdata.frame <- function(x, effect = seq_col(attr(x, "index")), w = NULL, cols = is.numeric, na.rm = TRUE, fill = TRUE,
+HDB.pdata.frame <- function(x, effect = "all", w = NULL, cols = is.numeric, na.rm = TRUE, fill = TRUE,
                             variable.wise = TRUE, stub = "HDB.", ...)
-  add_stub(fhdwithin.pdata.frame(if(is.null(cols)) x else colsubset(x, cols), effect, w, na.rm, fill, variable.wise, ..., means = TRUE), stub)
+  add_stub(fhdwithin.pdata.frame(fcolsubset(x, cols2intrmgn(which(attr(x, "names") %in% attr(findex(x), "nam")), cols, x)), effect, w, na.rm, fill, variable.wise, ..., means = TRUE), stub)
 
+HDB.list <- function(x, ...) HDB.data.frame(x, ...)
 
 fHDbetween <- function(x, ...) {
   message("Note that 'fHDbetween' was renamed to 'fhdbetween'. The S3 generic will not be removed anytime soon, but please use updated function names in new code, see help('collapse-renamed')")
