@@ -5,8 +5,8 @@ flast <- function(x, ...) UseMethod("flast") # , x
 
 flast.default <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names = TRUE, ...) {
   if(is.matrix(x) && !inherits(x, "matrix")) return(flast.matrix(x, g, TRA, na.rm, use.g.names, ...))
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(TRA)) {
+    if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) return(.Call(C_flast,x,0L,0L,na.rm))
     if(is.atomic(g)) {
       if(use.g.names) {
@@ -22,19 +22,14 @@ flast.default <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names = T
     if(use.g.names) return(`names<-`(.Call(C_flast,x,g[[1L]],g[[2L]],na.rm), GRPnames(g)))
     return(.Call(C_flast,x,g[[1L]],g[[2L]],na.rm))
   }
-  if(is.null(g)) return(.Call(Cpp_TRA,x,.Call(C_flast,x,0L,0L,na.rm),0L,TtI(TRA)))
-  if(is.atomic(g)) {
-    if(is.nmfactor(g)) return(.Call(Cpp_TRA,x,.Call(C_flast,x,fnlevels(g),g,na.rm),g,TtI(TRA)))
-    g <- qG(g, na.exclude = FALSE)
-    return(.Call(Cpp_TRA,x,.Call(C_flast,x,attr(g,"N.groups"),g,na.rm),g,TtI(TRA)))
-  }
-  if(!is_GRP(g)) g <- GRP.default(g, return.groups = FALSE, call = FALSE)
-  .Call(Cpp_TRA,x,.Call(C_flast,x,g[[1L]],g[[2L]],na.rm),g[[2L]],TtI(TRA))
+  if(is.null(g)) return(TRAC(x,.Call(C_flast,x,0L,0L,na.rm),0L,TRA, ...))
+  g <- G_guo(g)
+  TRAC(x,.Call(C_flast,x,g[[1L]],g[[2L]],na.rm),g[[2L]],TRA, ...)
 }
 
 flast.matrix <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names = TRUE, drop = TRUE, ...) {
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(TRA)) {
+    if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) return(.Call(C_flastm,x,0L,0L,na.rm,drop))
     if(is.atomic(g)) {
       if(use.g.names) {
@@ -50,19 +45,14 @@ flast.matrix <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names = TR
     if(use.g.names) return(`dimnames<-`(.Call(C_flastm,x,g[[1L]],g[[2L]],na.rm,FALSE), list(GRPnames(g), dimnames(x)[[2L]])))
     return(.Call(C_flastm,x,g[[1L]],g[[2L]],na.rm,FALSE))
   }
-  if(is.null(g)) return(.Call(Cpp_TRAm,x,.Call(C_flastm,x,0L,0L,na.rm,TRUE),0L,TtI(TRA)))
-  if(is.atomic(g)) {
-    if(is.nmfactor(g)) return(.Call(Cpp_TRAm,x,.Call(C_flastm,x,fnlevels(g),g,na.rm,FALSE),g,TtI(TRA)))
-    g <- qG(g, na.exclude = FALSE)
-    return(.Call(Cpp_TRAm,x,.Call(C_flastm,x,attr(g,"N.groups"),g,na.rm,FALSE),g,TtI(TRA)))
-  }
-  if(!is_GRP(g)) g <- GRP.default(g, return.groups = FALSE, call = FALSE)
-  .Call(Cpp_TRAm,x,.Call(C_flastm,x,g[[1L]],g[[2L]],na.rm,FALSE),g[[2L]],TtI(TRA))
+  if(is.null(g)) return(TRAmC(x,.Call(C_flastm,x,0L,0L,na.rm,TRUE),0L,TRA, ...))
+  g <- G_guo(g)
+  TRAmC(x,.Call(C_flastm,x,g[[1L]],g[[2L]],na.rm,FALSE),g[[2L]],TRA, ...)
 }
 
 flast.data.frame <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names = TRUE, drop = TRUE, ...) {
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   if(is.null(TRA)) {
+    if(!missing(...)) unused_arg_action(match.call(), ...)
     if(is.null(g)) if(drop) return(unlist(.Call(C_flastl,x,0L,0L,na.rm))) else return(.Call(C_flastl,x,0L,0L,na.rm))
     if(is.atomic(g)) {
       if(use.g.names && !inherits(x, "data.table")) {
@@ -79,21 +69,14 @@ flast.data.frame <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names
       return(setRnDF(.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm), groups))
     return(.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm))
   }
-  if(is.null(g)) return(.Call(Cpp_TRAl,x,.Call(C_flastl,x,0L,0L,na.rm),0L,TtI(TRA)))
-  if(is.atomic(g)) {
-    if(is.nmfactor(g)) return(.Call(Cpp_TRAl,x,.Call(C_flastl,x,fnlevels(g),g,na.rm),g,TtI(TRA)))
-    g <- qG(g, na.exclude = FALSE)
-    return(.Call(Cpp_TRAl,x,.Call(C_flastl,x,attr(g,"N.groups"),g,na.rm),g,TtI(TRA)))
-  }
-  if(!is_GRP(g)) g <- GRP.default(g, return.groups = FALSE, call = FALSE)
-  .Call(Cpp_TRAl,x,.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm),g[[2L]],TtI(TRA))
+  if(is.null(g)) return(TRAlC(x,.Call(C_flastl,x,0L,0L,na.rm),0L,TRA, ...))
+  g <- G_guo(g)
+  TRAlC(x,.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm),g[[2L]],TRA, ...)
 }
 
-flast.list <- function(x, g = NULL, TRA = NULL, na.rm = TRUE, use.g.names = TRUE, drop = TRUE, ...)
-  flast.data.frame(x, g, TRA, na.rm, use.g.names, drop, ...)
+flast.list <- function(x, ...) flast.data.frame(x, ...)
 
 flast.grouped_df <- function(x, TRA = NULL, na.rm = TRUE, use.g.names = FALSE, keep.group_vars = TRUE, ...) {
-  if(!missing(...)) unused_arg_action(match.call(), ...)
   g <- GRP.grouped_df(x, call = FALSE)
   nam <- attr(x, "names")
   gn <- which(nam %in% g[[5L]])
@@ -103,6 +86,7 @@ flast.grouped_df <- function(x, TRA = NULL, na.rm = TRUE, use.g.names = FALSE, k
     ax <- attributes(x)
     attributes(x) <- NULL
     if(nTRAl) {
+      if(!missing(...)) unused_arg_action(match.call(), ...)
       ax[["groups"]] <- NULL
       ax[["class"]] <- fsetdiff(ax[["class"]], c("GRP_df", "grouped_df"))
       ax[["row.names"]] <- if(use.g.names) GRPnames(g) else .set_row_names(g[[1L]])
@@ -119,9 +103,9 @@ flast.grouped_df <- function(x, TRA = NULL, na.rm = TRUE, use.g.names = FALSE, k
       } else return(setAttributes(.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm), ax))
     } else if(keep.group_vars) {
       ax[["names"]] <- c(nam[gn], nam[-gn])
-      return(setAttributes(c(x[gn],.Call(Cpp_TRAl,x[-gn],.Call(C_flastl,x[-gn],g[[1L]],g[[2L]],na.rm),g[[2L]],TtI(TRA))), ax))
+      return(setAttributes(c(x[gn],TRAlC(x[-gn],.Call(C_flastl,x[-gn],g[[1L]],g[[2L]],na.rm),g[[2L]],TRA, ...)), ax))
     }
     ax[["names"]] <- nam[-gn]
-    return(setAttributes(.Call(Cpp_TRAl,x[-gn],.Call(C_flastl,x[-gn],g[[1L]],g[[2L]],na.rm),g[[2L]],TtI(TRA)), ax))
-  } else return(.Call(Cpp_TRAl,x,.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm),g[[2L]],TtI(TRA)))
+    return(setAttributes(TRAlC(x[-gn],.Call(C_flastl,x[-gn],g[[1L]],g[[2L]],na.rm),g[[2L]],TRA, ...), ax))
+  } else return(TRAlC(x,.Call(C_flastl,x,g[[1L]],g[[2L]],na.rm),g[[2L]],TRA, ...))
 }
