@@ -49,7 +49,9 @@ ss <- function(x, i, j) {
   }
   rn <- attr(x, "row.names")
   if(is.numeric(rn) || is.null(rn) || rn[1L] == "1") return(.Call(C_subsetDT, x, i, j, checkrows))
-  return(`attr<-`(.Call(C_subsetDT, x, i, j, checkrows), "row.names", rn[i]))
+  res <- .Call(C_subsetDT, x, i, j, checkrows)
+  attr(res, "row.names") <- .Call(C_subsetVector, rn, i, checkrows)
+  res
 }
 
 fsubset.data.frame <- function(.x, subset, ...) {
@@ -81,8 +83,69 @@ fsubset.data.frame <- function(.x, subset, ...) {
   } else if(is.numeric(r)) r <- as.integer(r) else
     stop("subset needs to be an expression evaluating to logical(nrow(.x)) or integer")
   rn <- attr(.x, "row.names")
-  if(is.numeric(rn) || is.null(rn) || rn[1L] == "1") return(.Call(C_subsetDT, .x, r, vars, checkrows))
-  return(`attr<-`(.Call(C_subsetDT, .x, r, vars, checkrows), "row.names", rn[r]))
+  res <- .Call(C_subsetDT, .x, r, vars, checkrows)
+  if(is.numeric(rn) || is.null(rn) || rn[1L] == "1") return(res)
+  attr(res, "row.names") <- .Call(C_subsetVector, rn, r, checkrows)
+  res
+}
+
+
+fsubset.pseries <- function(.x, subset, ..., drop.index.levels = "id") {
+  if(is.array(.x)) stop("fsubset does not support pseries matrices")
+  if(!missing(...)) unused_arg_action(match.call(), ...)
+  checkrows <- TRUE
+  if(!is.integer(subset)) {
+    if(is.numeric(subset)) subset <- as.integer(subset) else if(is.logical(subset)) {
+      subset <- which(subset)
+      if(length(subset) == length(.x)) return(.x)
+      checkrows <- FALSE
+    } else stop("subset needs to be integer or logical")
+  }
+  res <- .Call(C_subsetVector, .x, subset, checkrows)
+  if(length(names(.x))) names(res) <- .Call(C_subsetVector, names(.x), subset, checkrows)
+  index <- findex(.x)
+  index_ss <- droplevels_index(.Call(C_subsetDT, index, subset, seq_along(unclass(index)), checkrows), drop.index.levels)
+  attr(res, if(inherits(.x, "indexed_series")) "index_df" else "index") <- index_ss
+  res
+}
+
+# Exact same code as .data.frame, just adding a block to deal with the index
+fsubset.pdata.frame <- function(.x, subset, ..., drop.index.levels = "id") {
+  r <- eval(substitute(subset), .x, parent.frame()) # Needs to be placed above any column renaming
+  if(missing(...)) vars <- seq_along(unclass(.x)) else {
+    ix <- seq_along(unclass(.x))
+    nl <- `names<-`(as.vector(ix, "list"), attr(.x, "names"))
+    vars <- eval(substitute(c(...)), nl, parent.frame())
+    nam_vars <- names(vars)
+    if(is.integer(vars)) {
+      if(any(vars < 0L)) vars <- ix[vars]
+    } else {
+      if(is.character(vars)) vars <- ckmatch(vars, names(nl)) else if(is.numeric(vars)) {
+        vars <- if(any(vars < 0)) ix[vars] else as.integer(vars)
+      } else stop("... needs to be comma separated column names, or column indices")
+    }
+    if(length(nam_vars)) {
+      nonmiss <- nzchar(nam_vars)
+      attr(.x, "names")[vars[nonmiss]] <- nam_vars[nonmiss]
+    }
+  }
+  checkrows <- TRUE
+  if(is.logical(r)) {
+    nr <- fnrow2(.x)
+    if(length(r) != nr) stop("subset needs to be an expression evaluating to logical(nrow(.x)) or integer") # which(r & !is.na(r)) not needed !
+    r <- which(r)
+    if(length(r) == nr) if(missing(...)) return(.x) else return(.Call(C_subsetCols, .x, vars, TRUE))
+    checkrows <- FALSE
+  } else if(is.numeric(r)) r <- as.integer(r) else
+    stop("subset needs to be an expression evaluating to logical(nrow(.x)) or integer")
+  rn <- attr(.x, "row.names")
+  res <- .Call(C_subsetDT, .x, r, vars, checkrows)
+  if(!(is.numeric(rn) || is.null(rn) || rn[1L] == "1")) attr(res, "row.names") <- .Call(C_subsetVector, rn, r, checkrows)
+  index <- findex(.x)
+  index_ss <- droplevels_index(.Call(C_subsetDT, index, r, seq_along(unclass(index)), checkrows), drop.index.levels)
+  if(inherits(.x, "indexed_frame")) return(reindex(res, index_ss))
+  attr(res, "index") <- index_ss
+  res
 }
 
 # Example:
@@ -188,8 +251,8 @@ settransform <- function(.data, ...)
 
 settfm <- settransform
 
-settransformv <- function(.data, vars, FUN, ..., apply = TRUE)
-  assign(as.character(substitute(.data)), ftransformv(.data, vars, FUN, ..., apply = apply), envir = parent.frame())
+settransformv <- function(.data, ...)
+  assign(as.character(substitute(.data)), ftransformv(.data, ...), envir = parent.frame())
 # eval.parent(substitute(.data <- get0("ftransformv", envir = getNamespace("collapse"))(.data, vars, FUN, ..., apply = apply)))
 
 settfmv <- settransformv
@@ -293,7 +356,8 @@ acr_get_cols <- function(.cols, d, nam, ce) {
   cols <- eval(.cols, nl, ce)
   # Needed for programming usage, because you can pass a variable that is null
   if(is.null(cols)) return(if(is.null(d[[".g_"]])) seq_along(nam) else seq_along(nam)[nam %!in% c(".g_", ".gsplit_", d[[".g_"]]$group.vars)])
-  return(cols2int(cols, d, nam)) # if(is.integer(cols)) cols else (you are checking against length(cols) in setup_across)
+  return(if(is.logical(cols)) which(cols) else cols2int(cols, d, nam)) # if .g_ etc. is added to data, length check for logical vectors will fail.
+  # if(is.integer(cols)) cols else (you are checking against length(cols) in setup_across)
 }
 
 # TODO: Implement for collap() ??
@@ -382,7 +446,7 @@ setup_across <- function(.cols, .fnsexp, .fns, .names, .apply, .transpose, .FFUN
   list(data = d,
        .data_ = .data_, # cols = cols,
        funs = fun,
-       aplvec = `names<-`(aplvec, names(fun)),
+       aplvec = aplvec,
        ce = ce,
        names = names)
 }
@@ -395,16 +459,16 @@ do_across <- function(.cols = NULL, .fns, ..., .names = NULL, .apply = "auto", .
   # nodots <- missing(...)
   # return(setup_across(substitute(.cols), substitute(.fns), .fns, .names, .apply, .FAST_FUN_MOPS))
   setup <- setup_across(substitute(.cols), substitute(.fns), .fns, .names, .apply, .transpose, .FAST_FUN_MOPS)
-  nf <- names(setup$funs)
+  seqf <- seq_along(setup$funs)
   names <- setup$names
-  # return(eval_funi(nf, ...))
-  # return(lapply(nf, eval_funi, ...))
-  if(length(nf) == 1L) {
-    res <- .eval_funi(nf, setup[[1L]], setup[[2L]], setup[[3L]], setup[[4L]], setup[[5L]], ...)  # eval_funi(nf, aplvec, funs, nodots, .data_, data, ce, ...)
+  # return(eval_funi(seqf, ...))
+  # return(lapply(seqf, eval_funi, ...))
+  if(length(seqf) == 1L) {
+    res <- .eval_funi(seqf, setup[[1L]], setup[[2L]], setup[[3L]], setup[[4L]], setup[[5L]], ...)  # eval_funi(seqf, aplvec, funs, nodots, .data_, data, ce, ...)
     # return(res)
   } else {
     # motivated by: fmutate(mtcars, across(cyl:vs, list(L, D, G), n = 1:3))
-    r <- lapply(nf, .eval_funi, setup[[1L]], setup[[2L]], setup[[3L]], setup[[4L]], setup[[5L]], ...) # do.call(lapply, c(list(nf, eval_funi), setup[1:5], list(...))) # lapply(nf, eval_funi, aplvec, funs, nodots, .data_, data, ce, ...)
+    r <- lapply(seqf, .eval_funi, setup[[1L]], setup[[2L]], setup[[3L]], setup[[4L]], setup[[5L]], ...) # do.call(lapply, c(list(seqf, eval_funi), setup[1:5], list(...))) # lapply(seqf, eval_funi, aplvec, funs, nodots, .data_, data, ce, ...)
     # return(r)
     if(isFALSE(.transpose) || (is.character(.transpose) && !all_eq(vlengths(r, FALSE)))) {
       # stop("reached here")
@@ -422,39 +486,40 @@ do_across <- function(.cols = NULL, .fns, ..., .names = NULL, .apply = "auto", .
 
 mutate_funi_simple <- function(i, data, .data_, funs, aplvec, ce, ...) { # g is unused here...
   .FUN_ <- funs[[i]]
+  nami <- names(funs)[i]
   if(aplvec[i]) {
     value <- if(missing(...)) lapply(unattrib(.data_), .FUN_) else
       do.call(lapply, c(list(unattrib(.data_), .FUN_), eval(substitute(list(...)), data, ce)), envir = ce) # eval(substitute(lapply(unattrib(.data_), .FUN_, ...)), c(list(.data_ = .data_), data), ce)
     names(value) <- names(.data_)
-  } else if(any(i == .FAST_STAT_FUN_POLD)) {
-    if(missing(...)) return(unclass(.FUN_(.data_, TRA = 1L))) # Old way: Not necessary to construct call.. return(unclass(eval(as.call(list(as.name(i), quote(.data_), TRA = 1L))))) # faster than substitute(.FUN_(.data_, TRA = 1L), list(.FUN_ = as.name(i)))
+  } else if(any(nami == .FAST_STAT_FUN_POLD)) {
+    if(missing(...)) return(unclass(.FUN_(.data_, TRA = 1L))) # Old way: Not necessary to construct call.. return(unclass(eval(as.call(list(as.name(nami), quote(.data_), TRA = 1L))))) # faster than substitute(.FUN_(.data_, TRA = 1L), list(.FUN_ = as.name(nami)))
     # if(any(...names() == "TRA")) # This down not work because it substitutes setup[[]] from mutate_across !!!
     #   return(unclass(eval(substitute(.FUN_(.data_, ...)), c(list(.data_ = .data_), data), ce)))
     # return(unclass(eval(substitute(.FUN_(.data_, ..., TRA = 1L)), c(list(.data_ = .data_), data), ce)))
-    fcal <- as.call(c(list(as.name(i), quote(.data_)), as.list(substitute(list(...))[-1L])))
+    fcal <- as.call(c(list(as.name(nami), quote(.data_)), as.list(substitute(list(...))[-1L])))
     if(is.null(fcal$TRA)) fcal$TRA <- 1L
     return(unclass(eval(fcal, c(list(.data_ = .data_), data), ce)))
   } else {
     value <- if(missing(...)) .FUN_(.data_) else
       do.call(.FUN_, c(list(.data_), eval(substitute(list(...)), data, ce)), envir = ce) # Object setup not found: eval(substitute(.FUN_(.data_, ...)), c(list(.data_ = .data_), data), ce)
     oldClass(value) <- NULL
-    if(any(i == .FAST_FUN_MOPS)) return(value) # small improvement for fast funs...
+    if(any(nami == .FAST_FUN_MOPS)) return(value) # small improvement for fast funs...
   }
   # return(unclass(r))
-  # fcal <- if(missing(...)) as.call(list(funs[[i]], quote(.data_))) else
-  #         as.call(c(list(funs[[i]], quote(.data_)), as.list(substitute(list(...))[-1L]))) # , parent.frame()
+  # fcal <- if(missing(...)) as.call(list(funs[[nami]], quote(.data_))) else
+  #         as.call(c(list(funs[[nami]], quote(.data_)), as.list(substitute(list(...))[-1L]))) # , parent.frame()
   #   # substitute(list(...), parent.frame())
-  #   # substitute(FUN(.data_, ...), list(FUN = funs[[i]], ...))
-  #   # as.call(substitute(list(funs[[i]], quote(.data_), ...)))
-  #   # substitute(FUN(.data_, ...), list(FUN = funs[[i]]))  #
-  # if(any(i == .FAST_STAT_FUN_POLD) && is.null(fcal$TRA)) fcal$TRA <- 1L
+  #   # substitute(FUN(.data_, ...), list(FUN = funs[[nami]], ...))
+  #   # as.call(substitute(list(funs[[nami]], quote(.data_), ...)))
+  #   # substitute(FUN(.data_, ...), list(FUN = funs[[nami]]))  #
+  # if(any(nami == .FAST_STAT_FUN_POLD) && is.null(fcal$TRA)) fcal$TRA <- 1L
   # fast functions have a data.frame method, thus can be applied simultaneously to all columns
   # return(fcal)
   # return(eval(fcal, c(list(.data_ = .data_), data), setup$ce))
   lv <- vlengths(value, FALSE)
   nr <- length(data[[1L]])
-  if(all(lv == nr)) return(value)
-  if(all(lv == 1L)) return(lapply(value, alloc, nr))
+  if(allv(lv, nr)) return(value)
+  if(allv(lv, 1L)) return(lapply(value, alloc, nr))
   stop("Without groups, NROW(value) must either be 1 or nrow(.data)")
 }
 
@@ -467,6 +532,7 @@ dots_apply_grouped <- function(d, g, f, dots) {
       asl <- lapply(dots[ln], gsplit, g)
       if(length(dots) > length(ln)) {
         mord <- dots[-ln]
+        # TODO: use .mapply() everywhere... if does not bump up R dependency??
         FUN <- function(x) do.call(mapply, c(list(f, gsplit(x, g), SIMPLIFY = FALSE, USE.NAMES = FALSE, MoreArgs = mord), asl))
       } else
         FUN <- function(x) do.call(mapply, c(list(f, gsplit(x, g), SIMPLIFY = FALSE, USE.NAMES = FALSE), asl))
@@ -517,17 +583,18 @@ dots_apply_grouped_bulk <- function(d, g, f, dots) {
 mutate_funi_grouped <- function(i, data, .data_, funs, aplvec, ce, ...) {
   g <- data[[".g_"]]
   .FUN_ <- funs[[i]]
+  nami <- names(funs)[i]
   apli <- aplvec[i]
   if(apli) {
     value <- if(missing(...)) lapply(unattrib(.data_), copysplaplfun, g, .FUN_) else
              dots_apply_grouped(.data_, g, .FUN_, eval(substitute(list(...)), data, ce)) # Before: do.call(lapply, c(list(unattrib(.data_), copysplaplfun, g, .FUN_), eval(substitute(list(...)), data, ce)), envir = ce)
-  } else if(any(i == .FAST_STAT_FUN_POLD)) {
+  } else if(any(nami == .FAST_STAT_FUN_POLD)) {
     if(missing(...)) return(unclass(.FUN_(.data_, g = g, TRA = 1L)))
-    fcal <- as.call(c(list(as.name(i), quote(.data_), g = quote(.g_)), as.list(substitute(list(...))[-1L])))
+    fcal <- as.call(c(list(as.name(nami), quote(.data_), g = quote(.g_)), as.list(substitute(list(...))[-1L])))
     if(is.null(fcal$TRA)) fcal$TRA <- 1L
     return(unclass(eval(fcal, c(list(.data_ = .data_), data), ce)))
-  } else if(any(i == .FAST_FUN_MOPS)) {
-    if(any(i == .OPERATOR_FUN)) {
+  } else if(any(nami == .FAST_FUN_MOPS)) {
+    if(any(nami == .OPERATOR_FUN)) {
       value <- if(missing(...)) .FUN_(.data_, by = g) else
         do.call(.FUN_, c(list(.data_, by = g), eval(substitute(list(...)), data, ce)), envir = ce)
     } else {
@@ -542,13 +609,20 @@ mutate_funi_grouped <- function(i, data, .data_, funs, aplvec, ce, ...) {
     oldClass(value) <- NULL
   }
   lv <- vlengths(value, FALSE)
-  nr <- length(data[[1L]])
-  if(all(lv == nr)) { # Improve efficiency here??
-    if(!isTRUE(g$ordered[2L])) value <- lapply(value, greorder, g)
+  nr <- length(g[[2L]])
+  if(allv(lv, nr)) {
+    if(!isTRUE(g$ordered[2L])) {
+      if(length(value) < 4L) { # optimal?
+        value <- lapply(value, function(x, g) .Call(C_greorder, x, g), g)
+      } else {
+        ind <- .Call(C_greorder, seq_len(nr), g)
+        value <- .Call(C_subsetDT, value, ind, seq_along(value), FALSE)
+      }
+    }
     if(apli) names(value) <- names(.data_)
     return(value)
   }
-  if(!all(lv == g[[1L]])) stop("With groups, NROW(value) must either be ng or nrow(.data)")
+  if(!allv(lv, g[[1L]])) stop("With groups, NROW(value) must either be ng or nrow(.data)")
   if(apli) names(value) <- names(.data_)
   return(.Call(C_subsetDT, value, g[[2L]], seq_along(value), FALSE))
 }
@@ -606,8 +680,8 @@ fmutate <- function(.data, ..., .keep = "all") {
         } else {
           r <- do_grouped_expr(ei, eiv, .data, g, pe)
           .data[[nam[i]]] <- if(length(r) == g[[1L]])
-               .Call(C_subsetVector, r, g[[2L]], FALSE) else # .Call(Cpp_TRA, .data[[v]], r, g[[2L]], 1L) # Faster than simple subset r[g[[2L]] ??]
-               greorder(r, g) # r[forder.int(forder.int(g[[2L]]))] # Seems twice is necessary...
+               .Call(C_subsetVector, r, g[[2L]], FALSE) else # .Call(C_TRA, .data[[v]], r, g[[2L]], 1L) # Faster than simple subset r[g[[2L]] ??]
+               .Call(C_greorder, r, g) # r[forder.int(forder.int(g[[2L]]))] # Seems twice is necessary...
         }
       }
     }
