@@ -26,10 +26,12 @@ SEXP sym_sorted;
 // SEXP sym_starts;
 // SEXP char_starts;
 SEXP sym_index;
+SEXP sym_index_df;
 SEXP sym_inherits;
 SEXP sym_sf_column;
 SEXP SelfRefSymbol;
 SEXP sym_datatable_locked;
+// SEXP sym_collapse_DT_alloccol;
 
 double NA_INT64_D;
 long long NA_INT64_LL;
@@ -139,10 +141,12 @@ SEXP collapse_init(SEXP mess) // void SEXP mess DllInfo *info
   sym_sorted  = install("sorted");
   // sym_maxgrpn = install("maxgrpn");
   sym_index   = install("index");
+  sym_index_df   = install("index_df");
   sym_inherits = install("inherits");
   sym_sf_column = install("sf_column");
   SelfRefSymbol = install(".internal.selfref");
   sym_datatable_locked = install(".data.table.locked");
+  // sym_collapse_DT_alloccol = install("collapse_DT_alloccol");
 
   return mess;
 }
