@@ -1,8 +1,10 @@
-// [[Rcpp::plugins(cpp11)]]
 #include <numeric>
-#define STRICT_R_HEADERS
+// #define STRICT_R_HEADERS // Now defined globally in Makevars
 #include <cfloat>
 #include <Rcpp.h>
+#ifdef _OPENMP
+#include <omp.h>
+#endif
 using namespace Rcpp;
 
 extern "C" {
@@ -15,7 +17,8 @@ auto nisnan = [](double x) { return x == x; };
 
 //[[Rcpp::export]]
 NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const IntegerVector& g = 0,
-                      const SEXP& gs = R_NilValue, const SEXP& w = R_NilValue, bool narm = true, int ret = 1) {
+                      const SEXP& gs = R_NilValue, const SEXP& w = R_NilValue, bool narm = true,
+                      int ret = 1, int nthreads = 1) {
   int l = x.size();
   if(l < 1) return x; // Prevents seqfault for numeric(0) #101
 
@@ -45,40 +48,55 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
         int sz = pend - xd.begin(), nth = lower ? (sz-1)*Q : sz*Q; // return NumericVector::create(sz, (sz-1)*Q, sz*Q-1, sz*Q);
         if(sz == 0) return Rf_ScalarReal(x[0]);
         std::nth_element(xd.begin(), xd.begin()+nth, pend);
-        out = (tiesmean && sz%2 == 0) ? (xd[nth] + *(std::min_element(xd.begin()+nth+1, pend)))*0.5 : xd[nth];
+        out[0] = (tiesmean && sz%2 == 0) ? (xd[nth] + *(std::min_element(xd.begin()+nth+1, pend)))*0.5 : xd[nth];
       } else {
-        for(int i = 0; i != l; ++i) if(isnan2(x[i])) return Rf_ScalarReal(x[i]);
-        NumericVector xd = Rf_duplicate(x);
-        int nth = lower ? (l-1)*Q : l*Q;
-        std::nth_element(xd.begin(), xd.begin()+nth, xd.end());
-        out = (tiesmean && l%2 == 0) ? (xd[nth] + *(std::min_element(xd.begin()+nth+1, xd.end())))*0.5 : xd[nth];
+        for(int i = 0; i != l; ++i) {
+          if(isnan2(x[i])) {
+            out[0] = x[i];
+            break;
+          }
+        }
+        if(nisnan(out[0])) {
+          NumericVector xd = Rf_duplicate(x);
+          int nth = lower ? (l-1)*Q : l*Q;
+          std::nth_element(xd.begin(), xd.begin()+nth, xd.end());
+          out[0] = (tiesmean && l%2 == 0) ? (xd[nth] + *(std::min_element(xd.begin()+nth+1, xd.end())))*0.5 : xd[nth];
+        }
       }
+      if(ATTRIB(x) != R_NilValue && !(Rf_isObject(x) && Rf_inherits(x, "ts")))
+        Rf_copyMostAttrib(x, out);
       return out;
     } else { // with groups
       if(l != g.size()) stop("length(g) must match length(x)");
+      if(nthreads > ng) nthreads = ng;
       int ngp = ng+1;
       std::vector<std::vector<double> > gmap(ngp);
       std::vector<int> gcount(ngp);
       if(Rf_isNull(gs)) {
         for(int i = 0; i != l; ++i) ++gcount[g[i]];
-        for(int i = 1; i != ngp; ++i) {
-          if(gcount[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
-          gmap[i] = std::vector<double> (gcount[i]);
-          gcount[i] = 0;
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 1; i < ngp; ++i) {
+          // if(gcount[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
+          if(gcount[i] > 0) {
+            gmap[i] = std::vector<double> (gcount[i]);
+            gcount[i] = 0;
+          }
         }
       } else {
         IntegerVector gsv = gs;
         if(ng != gsv.size()) stop("ng must match length(gs)");
-        for(int i = 0; i != ng; ++i) {
-          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
-          gmap[i+1] = std::vector<double> (gsv[i]);
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 0; i < ng; ++i) {
+          // if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
+          if(gsv[i] > 0) gmap[i+1] = std::vector<double> (gsv[i]);
         }
       }
 
       if(narm) {
         NumericVector out(ng, NA_REAL);
         for(int i = 0; i != l; ++i) if(nisnan(x[i])) gmap[g[i]][gcount[g[i]]++] = x[i];
-        for(int i = 1; i != ngp; ++i) {
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 1; i < ngp; ++i) {
           if(gcount[i] != 0) {
             int n = gcount[i], nth = lower ? (n-1)*Q : n*Q;
             auto begin = gmap[i].begin(), mid = begin + nth, end = begin + n;
@@ -86,7 +104,8 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
             out[i-1] = (tiesmean && n%2 == 0) ? (*(mid) + *(std::min_element(mid+1, end)))*0.5 : *(mid);
           }
         }
-        if(!Rf_isObject(x)) Rf_copyMostAttrib(x, out);
+        if(ATTRIB(x) != R_NilValue && !(Rf_isObject(x) && Rf_inherits(x, "ts")))
+          Rf_copyMostAttrib(x, out);
         return out;
       } else {
         NumericVector out(ng);
@@ -102,14 +121,16 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
             gmap[g[i]][gcount[g[i]]++] = x[i];
           }
         }
-        for(int i = 0; i != ng; ++i) {
-          if(isnan2(out[i])) continue;
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 0; i < ng; ++i) {
+          if(isnan2(out[i]) || gcount[i+1] == 0) continue;
           int n = gcount[i+1], nth = lower ? (n-1)*Q : n*Q;
           auto begin = gmap[i+1].begin(), mid = begin + nth, end = begin + n;
           std::nth_element(begin, mid, end);
           out[i] = (tiesmean && n%2 == 0) ? (*(mid) + *(std::min_element(mid+1, end)))*0.5 : *(mid);
         }
-        if(!Rf_isObject(x)) Rf_copyMostAttrib(x, out);
+        if(ATTRIB(x) != R_NilValue && !(Rf_isObject(x) && Rf_inherits(x, "ts")))
+          Rf_copyMostAttrib(x, out);
         return out;
       }
     }
@@ -118,17 +139,23 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
     if(l != wg.size()) stop("length(w) must match length(x)");
     IntegerVector o = no_init_vector(l);
     int *ord = INTEGER(o);
-    Cdoubleradixsort(ord, TRUE, FALSE, x); // starts from 1
+    Cdoubleradixsort(ord, TRUE, FALSE, wrap(x)); // starts from 1
 
     if(ng == 0) {
-      double wsumQ = 0, wsum = wg[o[0]-1];
+      double wsumQ = 0, wsum = wg[o[0]-1], res = DBL_MIN;
       int k = 1;
       if(narm) {
-        for(int i = 0; i != l; ++i) if(nisnan(x[i])) wsumQ += wg[i]; //  && nisnan(wg[i])
-        if(wsumQ == 0) return Rf_ScalarReal(NA_REAL);
+        for(int i = 0; i < l; ++i) if(nisnan(x[i])) wsumQ += wg[i]; //  && nisnan(wg[i])
+        if(wsumQ == 0) {
+          res = NA_REAL;
+          goto end;
+        }
         wsumQ *= Q;
       } else {
-        if(isnan2(x[o[l-1]-1])) return Rf_ScalarReal(NA_REAL);
+        if(isnan2(x[o[l-1]-1]))  {
+          res = NA_REAL;
+          goto end;
+        }
         wsumQ = std::accumulate(wg.begin(), wg.end(), 0.0) * Q;
       }
       if(isnan2(wsumQ)) stop("Missing weights in order statistics are currently only supported if x is also missing"); // return Rf_ScalarReal(NA_REAL);
@@ -140,13 +167,19 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
             out += x[o[k++]-1];
             ++n;
           }
-          return Rf_ScalarReal((out + x[o[k]-1]) / n);
+          res = (out + x[o[k]-1]) / n;
         }
       } else {
         while(wsum <= wsumQ) wsum += wg[o[k++]-1];
       }
-      return Rf_ScalarReal(x[o[k-1]-1]);
-    } else { // with groups and weights
+      if(res == DBL_MIN) res = x[o[k-1]-1];
+      end:;
+      if(ATTRIB(x) != R_NilValue && !(Rf_isObject(x) && Rf_inherits(x, "ts"))) {
+        SEXP out = Rf_ScalarReal(res);
+        Rf_copyMostAttrib(x, out);
+        return out;
+      } else return Rf_ScalarReal(res);
+    } else { // with groups and weights: no efficient parallelism possible...
       if(l != g.size()) stop("length(g) must match length(x)");
       NumericVector wsumQ(ng), wsum(ng), out(ng, NA_REAL);
       if(narm) {
@@ -206,7 +239,8 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
           }
         }
       }
-      if(!Rf_isObject(x)) Rf_copyMostAttrib(x, out);
+      if(ATTRIB(x) != R_NilValue && !(Rf_isObject(x) && Rf_inherits(x, "ts")))
+        Rf_copyMostAttrib(x, out);
       return out;
     }
   }
@@ -219,7 +253,7 @@ NumericVector fnthCpp(const NumericVector& x, double Q = 0.5, int ng = 0, const
 //[[Rcpp::export]]
 SEXP fnthmCpp(const NumericMatrix& x, double Q = 0.5, int ng = 0, const IntegerVector& g = 0,
               const SEXP& gs = R_NilValue, const SEXP& w = R_NilValue, bool narm = true,
-              bool drop = true, int ret = 1) {
+              bool drop = true, int ret = 1, int nthreads = 1) {
   int l = x.nrow(), col = x.ncol();
   bool tiesmean, lower;
   if(Q <= 0 || Q == 1) stop("n needs to be between 0 and 1, or between 1 and nrow(x). Use fmin and fmax for minima and maxima.");
@@ -240,25 +274,27 @@ SEXP fnthmCpp(const NumericMatrix& x, double Q = 0.5, int ng = 0, const IntegerV
 
   if(Rf_isNull(w)) {
     if(ng == 0) {
+      if(nthreads > col) nthreads = col;
       NumericVector out = no_init_vector(col);
       if(narm) {
-        NumericVector column = no_init_vector(l);
-        auto begin = column.begin();
-        for(int j = col; j--; ) {
+        #pragma omp parallel for num_threads(nthreads)
+        for(int j = 0; j < col; ++j) {
           NumericMatrix::ConstColumn colj = x( _ , j);
-          auto pend = std::remove_copy_if(colj.begin(), colj.end(), begin, isnan2);
+          NumericVector column = no_init_vector(l); // without multithreading this could be taken out of the loop, see previous version of the code.
+          double *begin = column.begin(), *pend = std::remove_copy_if(colj.begin(), colj.end(), begin, isnan2);
           int sz = pend - begin, nth = lower ? (sz-1)*Q : sz*Q;
           if(sz == 0) {
             out[j] = colj[0];
-            continue;
+          } else {
+            std::nth_element(begin, begin+nth, pend);
+            out[j] = (tiesmean && sz%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, pend)))*0.5 : column[nth];
           }
-          std::nth_element(begin, begin+nth, pend);
-          out[j] = (tiesmean && sz%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, pend)))*0.5 : column[nth];
         }
       } else {
         int nth = lower ? (l-1)*Q : l*Q;
         bool tm = tiesmean && l%2 == 0;
-        for(int j = col; j--; ) {
+        #pragma omp parallel for num_threads(nthreads)
+        for(int j = 0; j < col; ++j) {
           {
             NumericMatrix::ConstColumn colj = x( _ , j);
             for(int i = 0; i != l; ++i) {
@@ -283,33 +319,37 @@ SEXP fnthmCpp(const NumericMatrix& x, double Q = 0.5, int ng = 0, const IntegerV
       return out;
     } else { // with groups
       if(l != g.size()) stop("length(g) must match nrow(x)");
+      if(nthreads > ng) nthreads = ng;
       int ngp = ng+1;
       std::vector<std::vector<double> > gmap(ngp);
       std::vector<int> gcount(ngp);
       if(Rf_isNull(gs)) {
         for(int i = 0; i != l; ++i) ++gcount[g[i]];
-        for(int i = 1; i != ngp; ++i) {
-          if(gcount[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
-          gmap[i] = std::vector<double> (gcount[i]);
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 1; i < ngp; ++i) {
+          // if(gcount[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
+          if(gcount[i] > 0) gmap[i] = std::vector<double> (gcount[i]);
         }
       } else {
         IntegerVector gsv = gs;
         if(ng != gsv.size()) stop("ng must match length(gs)");
-        for(int i = 0; i != ng; ++i) {
-          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
-          gmap[i+1] = std::vector<double> (gsv[i]);
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 0; i < ng; ++i) {
+          // if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
+          if(gsv[i] > 0) gmap[i+1] = std::vector<double> (gsv[i]);
         }
       }
 
       if(narm) {
         NumericMatrix out = no_init_matrix(ng, col);
-        std::fill(out.begin(), out.end(), NA_REAL);
+        std::fill(out.begin(), out.end(), NA_REAL); // parallelize??
         for(int j = col; j--; ) {
           NumericMatrix::ConstColumn column = x( _ , j);
           NumericMatrix::Column nthj = out( _ , j);
           gcount.assign(ngp, 0);
           for(int i = 0; i != l; ++i) if(nisnan(column[i])) gmap[g[i]][gcount[g[i]]++] = column[i];
-          for(int i = 1; i != ngp; ++i) {
+          #pragma omp parallel for num_threads(nthreads)
+          for(int i = 1; i < ngp; ++i) {
             if(gcount[i] != 0) {
               int n = gcount[i], nth = lower ? (n-1)*Q : n*Q;
               auto begin = gmap[i].begin(), mid = begin + nth, end = begin + n;
@@ -339,8 +379,9 @@ SEXP fnthmCpp(const NumericMatrix& x, double Q = 0.5, int ng = 0, const IntegerV
               gmap[g[i]][gcount[g[i]]++] = column[i];
             }
           }
-          for(int i = 0; i != ng; ++i) {
-            if(isnan2(nthj[i])) continue;
+          #pragma omp parallel for num_threads(nthreads)
+          for(int i = 0; i < ng; ++i) {
+            if(isnan2(nthj[i]) || gcount[i+1] == 0) continue;
             int n = gcount[i+1], nth = lower ? (n-1)*Q : n*Q;
             auto begin = gmap[i+1].begin(), mid = begin + nth, end = begin + n;
             std::nth_element(begin, mid, end);
@@ -407,13 +448,13 @@ SEXP fnthmCpp(const NumericMatrix& x, double Q = 0.5, int ng = 0, const IntegerV
         out[j] = column[o[k-1]-1];
       }
       // outnth:
-        if(drop) Rf_setAttrib(out, R_NamesSymbol, colnames(x));
-        else {
-          Rf_dimgets(out, Dimension(1, col));
-          colnames(out) = colnames(x);
-          if(!Rf_isObject(x)) Rf_copyMostAttrib(x, out);
-        }
-        return out;
+      if(drop) Rf_setAttrib(out, R_NamesSymbol, colnames(x));
+      else {
+        Rf_dimgets(out, Dimension(1, col));
+        colnames(out) = colnames(x);
+        if(!Rf_isObject(x)) Rf_copyMostAttrib(x, out);
+      }
+      return out;
     } else { // with groups and weights
       if(l != g.size()) stop("length(g) must match nrow(x)");
       int gi, oi;
@@ -487,12 +528,36 @@ SEXP fnthmCpp(const NumericMatrix& x, double Q = 0.5, int ng = 0, const IntegerV
 
 
 
+// double median_narm_single(const NumericVector& colj, NumericVector& column, const bool lower, const bool tiesmean, const double Q) {
+//   double *begin = column.begin(), *pend = std::remove_copy_if(colj.begin(), colj.end(), begin, isnan2);
+//   int sz = pend - begin, nth = lower ? (sz-1)*Q : sz*Q;
+//   if(sz == 0) return colj[0];
+//   std::nth_element(begin, begin+nth, pend);
+//   return (tiesmean && sz%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, pend)))*0.5 : column[nth];
+// }
 
+double median_narm(const NumericVector& colj, const bool lower, const bool tiesmean, const double Q) {
+  NumericVector column = no_init_vector(colj.size());
+  double *begin = column.begin(), *pend = std::remove_copy_if(colj.begin(), colj.end(), begin, isnan2);
+  int sz = pend - begin, nth = lower ? (sz-1)*Q : sz*Q;
+  if(sz == 0) return colj[0];
+  std::nth_element(begin, begin+nth, pend);
+  return (tiesmean && sz%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, pend)))*0.5 : column[nth];
+}
+
+double median_keepna(const NumericVector& colj, const bool lower, const bool tiesmean, const double Q) {
+  int row = colj.size(), nth = lower ? (row-1)*Q : row*Q;
+  for(int i = 0; i != row; ++i) if(isnan2(colj[i])) return NA_REAL;
+  NumericVector column = Rf_duplicate(colj);
+  double *begin = column.begin();
+  std::nth_element(begin, begin+nth, column.end());
+  return (tiesmean && row%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, column.end())))*0.5 : column[nth];
+}
 
 //[[Rcpp::export]]
 SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g = 0,
               const SEXP& gs = R_NilValue, const SEXP& w = R_NilValue, bool narm = true,
-              bool drop = true, int ret = 1) {
+              bool drop = true, int ret = 1, int nthreads = 1) {
   int l = x.size(), lx1 = Rf_length(x[0]);
   bool tiesmean, lower;
   if(Q <= 0 || Q == 1) stop("n needs to be between 0 and 1, or between 1 and nrow(x). Use fmin and fmax for minima and maxima.");
@@ -511,41 +576,39 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
     lower = ret != 3;
   }
 
+
   if(Rf_isNull(w)) {
+    // Needed with multithreading over columns...
+    for(int j = 0, tx; j < l; ++j) {
+      tx = TYPEOF(x[j]);
+      if(!(tx == REALSXP || tx == INTSXP || tx == LGLSXP))
+        stop("All columns of x need to be numeric");
+    }
     if(ng == 0) {
+      if(nthreads > l) nthreads = l;
       NumericVector out = no_init_vector(l);
       if(narm) {
-        for(int j = l; j--; ) {
-          NumericVector colj = x[j];
-          NumericVector column = no_init_vector(colj.size());
-          auto begin = column.begin();
-          auto pend = std::remove_copy_if(colj.begin(), colj.end(), begin, isnan2);
-          int sz = pend - begin, nth = lower ? (sz-1)*Q : sz*Q;
-          if(sz == 0) {
-            out[j] = colj[0];
-            continue;
+        if(nthreads == 1) {
+          NumericVector column = no_init_vector(lx1);
+          for(int j = 0; j < l; ++j) {
+            //out[j] = median_narm_single(x[j], column, lower, tiesmean, Q);
+            NumericVector colj = x[j];
+            double *begin = column.begin(), *pend = std::remove_copy_if(colj.begin(), colj.end(), begin, isnan2);
+            int sz = pend - begin, nth = lower ? (sz-1)*Q : sz*Q;
+            if(sz == 0) {
+              out[j] = colj[0];
+            } else {
+              std::nth_element(begin, begin+nth, pend);
+              out[j] = (tiesmean && sz%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, pend)))*0.5 : column[nth];
+            }
           }
-          std::nth_element(begin, begin+nth, pend);
-          out[j] = (tiesmean && sz%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, pend)))*0.5 : column[nth];
+        } else {
+          #pragma omp parallel for num_threads(nthreads)
+          for(int j = 0; j < l; ++j) out[j] = median_narm(x[j], lower, tiesmean, Q);
         }
       } else {
-        for(int j = l; j--; ) {
-          {
-            NumericVector colj = x[j];
-            int row = colj.size(), nth = lower ? (row-1)*Q : row*Q;
-            for(int i = 0; i != row; ++i) {
-              if(isnan2(colj[i])) {
-                out[j] = colj[i];
-                goto endloop;
-              }
-            }
-            NumericVector column = Rf_duplicate(colj);
-            auto begin = column.begin();
-            std::nth_element(begin, begin+nth, column.end());
-            out[j] = (tiesmean && row%2 == 0) ? (column[nth] + *(std::min_element(begin+nth+1, column.end())))*0.5 : column[nth];
-          }
-          endloop:;
-        }
+        #pragma omp parallel for num_threads(nthreads)
+        for(int j = 0; j < l; ++j) out[j] = median_keepna(x[j], lower, tiesmean, Q);
       }
       if(drop) {
         Rf_setAttrib(out, R_NamesSymbol, Rf_getAttrib(x, R_NamesSymbol));
@@ -563,22 +626,25 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
 
     } else { // with groups
       if(lx1 != g.size()) stop("length(g) must match nrow(x)");
+      if(nthreads > ng) nthreads = ng;
       List out(l);
       int ngp = ng+1;
       std::vector<std::vector<double> > gmap(ngp);
       std::vector<int> gcount(ngp);
       if(Rf_isNull(gs)) {
         for(int i = 0; i != lx1; ++i) ++gcount[g[i]];
-        for(int i = 1; i != ngp; ++i) {
-          if(gcount[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
-          gmap[i] = std::vector<double> (gcount[i]);
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 1; i < ngp; ++i) {
+          // if(gcount[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
+          if(gcount[i] > 0) gmap[i] = std::vector<double> (gcount[i]);
         }
       } else {
         IntegerVector gsv = gs;
         if(ng != gsv.size()) stop("ng must match length(gs)");
-        for(int i = 0; i != ng; ++i) {
-          if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
-          gmap[i+1] = std::vector<double> (gsv[i]);
+        #pragma omp parallel for num_threads(nthreads)
+        for(int i = 0; i < ng; ++i) {
+          // if(gsv[i] == 0) stop("Group size of 0 encountered. This is probably because of unused factor levels. Use fdroplevels(f) to drop them.");
+          if(gsv[i] > 0) gmap[i+1] = std::vector<double> (gsv[i]);
         }
       }
       if(narm) {
@@ -588,7 +654,8 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
           if(lx1 != column.size()) stop("length(g) must match nrow(x)");
           gcount.assign(ngp, 0);
           for(int i = 0; i != lx1; ++i) if(nisnan(column[i])) gmap[g[i]][gcount[g[i]]++] = column[i];
-          for(int i = 1; i != ngp; ++i) {
+          #pragma omp parallel for num_threads(nthreads)
+          for(int i = 1; i < ngp; ++i) {
             if(gcount[i] != 0) {
               int n = gcount[i], nth = lower ? (n-1)*Q : n*Q;
               auto begin = gmap[i].begin(), mid = begin + nth, end = begin + n;
@@ -617,8 +684,9 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
               gmap[g[i]][gcount[g[i]]++] = column[i];
             }
           }
-          for(int i = 0; i != ng; ++i) {
-            if(isnan2(nthj[i])) continue;
+          #pragma omp parallel for num_threads(nthreads)
+          for(int i = 0; i < ng; ++i) {
+            if(isnan2(nthj[i]) || gcount[i+1] == 0) continue;
             int n = gcount[i+1], nth = lower ? (n-1)*Q : n*Q;
             auto begin = gmap[i+1].begin(), mid = begin + nth, end = begin + n;
             std::nth_element(begin, mid, end);
@@ -652,7 +720,7 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
       for(int j = l; j--; ) {
         NumericVector column = x[j];
         if(lx1 != column.size()) stop("length(w) must match nrow(x)");
-        Cdoubleradixsort(ord, TRUE, FALSE, column); // starts from 1
+        Cdoubleradixsort(ord, TRUE, FALSE, wrap(column)); // starts from 1
         if(narm) {
           wsumQ = 0;
           for(int i = 0; i != lx1; ++i) if(nisnan(column[i])) wsumQ += wg[i]; //  && nisnan(wg[i])
@@ -688,19 +756,19 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
         out[j] = column[o[k-1]-1];
       }
       // outnth:
-        if(drop) {
-          Rf_setAttrib(out, R_NamesSymbol, Rf_getAttrib(x, R_NamesSymbol));
-          return out;
-        } else {
-          List outl(l);
-          for(int j = l; j--; ) {
-            outl[j] = out[j];
-            SHALLOW_DUPLICATE_ATTRIB(outl[j], x[j]);
-          }
-          SHALLOW_DUPLICATE_ATTRIB(outl, x);
-          Rf_setAttrib(outl, R_RowNamesSymbol, Rf_ScalarInteger(1));
-          return outl;
+      if(drop) {
+        Rf_setAttrib(out, R_NamesSymbol, Rf_getAttrib(x, R_NamesSymbol));
+        return out;
+      } else {
+        List outl(l);
+        for(int j = l; j--; ) {
+          outl[j] = out[j];
+          SHALLOW_DUPLICATE_ATTRIB(outl[j], x[j]);
         }
+        SHALLOW_DUPLICATE_ATTRIB(outl, x);
+        Rf_setAttrib(outl, R_RowNamesSymbol, Rf_ScalarInteger(1));
+        return outl;
+      }
     } else { // with groups and weights
       if(lx1 != g.size()) stop("length(w) must match length(g)");
       int gi, oi;
@@ -713,7 +781,7 @@ SEXP fnthlCpp(const List& x, double Q = 0.5, int ng = 0, const IntegerVector& g
       for(int j = l; j--; ) {
         NumericVector column = x[j];
         if(lx1 != column.size()) stop("length(w) must match nrow(x)");
-        Cdoubleradixsort(ord, TRUE, FALSE, column);
+        Cdoubleradixsort(ord, TRUE, FALSE, wrap(column));
         NumericVector wsum(ng), nthj(ng, NA_REAL);
         if(narm) {
           std::fill(wsumQ.begin(), wsumQ.end(), 0.0);
