@@ -5,18 +5,17 @@
 
 #include "kit.h"
 
-// TODO: Check if division hash is not faster, or use Rcpp IndexHash
-// TODO: Option to Preserva NA's ?
 
 // ****************************************
 // This function groups a single vector
 // ****************************************
 SEXP dupVecIndex(SEXP x) {
   const int n = length(x);
-  int K, tx = TYPEOF(x);
+  int K = 0, tx = TYPEOF(x), x_min = INT_MAX, x_max = INT_MIN, anyNA = 0;
   size_t M;
-  if(n >= INT_MAX) error("Length of 'x' is too large. (Long vector not supported yet)"); // 1073741824
+  // if(n >= INT_MAX) error("Length of 'x' is too large. (Long vector not supported yet)"); // 1073741824
   if (tx == STRSXP || tx == REALSXP || tx == CPLXSXP ) {
+    bigint:;
     const size_t n2 = 2U * (size_t) n;
     M = 256;
     K = 8;
@@ -25,51 +24,108 @@ SEXP dupVecIndex(SEXP x) {
       K++;
     }
   } else if(tx == INTSXP) {
-    if(isFactor(x)) {
+    if(isFactor(x) || inherits(x, "qG")) {
       tx = 1000;
-      M = (size_t)nlevels(x) + 2;
-    } else M = (size_t)n;
+      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, install("N.groups"))) + 2;
+      anyNA = !inherits(x, "na.included");
+    } else {
+      int *restrict p = INTEGER(x);
+      // Old:
+      if(n < 10 || NOGE(p[0], n) || NOGE(p[n/2], n) || NOGE(p[n-1], n)) {
+        // This loop is highly optimized...
+        for(int i = 0, x_tmp; i != n; ++i) {
+          x_tmp = p[i];
+          if(x_tmp > x_max) x_max = x_tmp;
+          if(x_tmp < x_min) {
+            if(x_tmp == NA_INTEGER) anyNA = 1;
+            else x_min = x_tmp;
+          }
+        }
+        x_max -= x_min;
+        if(++x_max > 3 * n) goto bigint;
+        M = (size_t)(x_max + 2);
+        if(x_min == 0 || x_min == 1) tx = 1000;
+        else x_max = NA_INTEGER;
+      } else M = (size_t)n;
+    }
   } else if (tx == LGLSXP) {
     M = 3;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
-  // memset(h, 0, M * sizeof(int)); // not needed??
+  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
   SEXP ans_i = PROTECT(allocVector(INTSXP, n));
-  int *pans_i = INTEGER(ans_i);
-  size_t id = 0, g = 0;
+  int *restrict pans_i = INTEGER(ans_i), g = 0;
+  size_t id = 0;
   switch (tx) {
   case LGLSXP:
   case 1000: // This is for factors or logical vectors where the size of the table is known
   {
-    const int *px = INTEGER(x);
-    for (int i = 0, j, k = (int)M-1; i != n; ++i) {
-      j = (px[i] == NA_INTEGER) ? k : px[i];
-      if(h[j]) pans_i[i] = h[j];
-      else pans_i[i] = h[j] = ++g;
+    const int *restrict px = INTEGER(x);
+    if(tx == 1000 && !anyNA) {
+      for(int i = 0, j; i != n; ++i) {
+        j = px[i];
+        if(h[j]) pans_i[i] = h[j];
+        else pans_i[i] = h[j] = ++g;
+      }
+    } else {
+      for(int i = 0, j, k = (int)M-1; i != n; ++i) {
+        j = (px[i] == NA_INTEGER) ? k : px[i];
+        if(h[j]) pans_i[i] = h[j];
+        else pans_i[i] = h[j] = ++g;
+      }
     }
   } break;
-  case INTSXP: { // Faster version based on division hash...
-    const int *px = INTEGER(x);
-    unsigned int iid = 0, nu = (unsigned)n;
-    for (int i = 0; i != n; ++i) {
-      iid = (unsigned)px[i];
-      if(iid >= nu) iid %= nu;
-      // iid = (xi < nu) ? xi : xi % nu; // HASH(px[i], K); // get the hash value of x[i]
-      while(h[iid]) { // Check if this hash value has been seen before
-        if(px[h[iid]-1] == px[i]) { // Get the element of x that produced his value. if x[i] is the same, assign it the same index.
-          pans_i[i] = pans_i[h[iid]-1]; // h[id];
-          goto ibl;
-        } // else, we move forward to the next slot, until we find an empty one... We need to keep checking against the values,
-          // because if we found the same value before, we would also have put it in another slot after the initial one with the same hash value.
-        if(++iid >= nu) iid %= nu; // # nocov
-      } // We put the index into the empty slot.
-      h[iid] = i + 1; // need + 1 because for zero the while loop gives false..
-      pans_i[i] = ++g; // h[id];
-      ibl:;
+  case INTSXP: {
+    const int *restrict px = INTEGER(x);
+    // Old:
+    if(x_max == INT_MIN && M == (size_t)n) { // Faster version based on division hash...
+      unsigned int iid = 0, nu = (unsigned)n;
+      for (int i = 0; i != n; ++i) {
+        iid = (unsigned)px[i];
+        if(iid >= nu) iid %= nu;
+        while(h[iid]) {
+          if(px[h[iid]-1] == px[i]) {
+            pans_i[i] = pans_i[h[iid]-1];
+            goto ibl;
+          }
+          if(++iid >= nu) iid %= nu;
+        }
+        h[iid] = i + 1; // need + 1 because for zero the while loop gives false..
+        pans_i[i] = ++g;
+        ibl:;
+      }
+    } else if(x_max == NA_INTEGER) { // fastver version based on range
+      x_min -= 1;
+      if(anyNA) {
+        for (int i = 0, j; i != n; ++i) {
+          j = (px[i] == NA_INTEGER) ? 0 : px[i]-x_min;
+          if(h[j]) pans_i[i] = h[j];
+          else pans_i[i] = h[j] = ++g;
+        }
+      } else {
+        for (int i = 0, j; i != n; ++i) {
+          j = px[i]-x_min;
+          if(h[j]) pans_i[i] = h[j];
+          else pans_i[i] = h[j] = ++g;
+        }
+      }
+    } else {
+      for (int i = 0; i != n; ++i) {
+        id = HASH(px[i], K);
+        while(h[id]) {
+          if(px[h[id]-1] == px[i]) {
+            pans_i[i] = pans_i[h[id]-1]; // h[id];
+            goto ibbl;
+          }
+          if(++id >= M) id %= M;
+        }
+        h[id] = i + 1;
+        pans_i[i] = ++g; // h[id];
+        ibbl:;
+      }
     }
   } break;
   case REALSXP: {
-    const double *px = REAL(x);
+    const double *restrict px = REAL(x);
     union uno tpv;
     for (int i = 0; i != n; ++i) {
       tpv.d = px[i]; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN : px[i]);
@@ -87,14 +143,13 @@ SEXP dupVecIndex(SEXP x) {
     }
   } break;
   case CPLXSXP: {
-    const Rcomplex *px = COMPLEX(x);
+    const Rcomplex *restrict px = COMPLEX(x);
     unsigned int u;
     union uno tpv;
     Rcomplex tmp;
     for (int i = 0; i != n; ++i) {
-      tmp.r = (px[i].r == 0.0) ? 0.0 : px[i].r;
-      tmp.i = (px[i].i == 0.0) ? 0.0 : px[i].i;
-      if (C_IsNA(tmp)) {
+      tmp = px[i];
+      if(C_IsNA(tmp)) {
         tmp.r = tmp.i = NA_REAL;
       } else if (C_IsNaN(tmp)) {
         tmp.r = tmp.i = R_NaN;
@@ -117,7 +172,7 @@ SEXP dupVecIndex(SEXP x) {
     }
   } break;
   case STRSXP: {
-    const SEXP *px = STRING_PTR(x);
+    const SEXP *restrict px = STRING_PTR(x);
     for (int i = 0; i != n; ++i) {
       id = HASH(((intptr_t) px[i] & 0xffffffff), K);
       while(h[id]) {
@@ -134,7 +189,8 @@ SEXP dupVecIndex(SEXP x) {
   } break;
   }
   Free(h);
-  setAttrib(ans_i, install("N.groups"), ScalarInteger(g));
+  SEXP ngroups_sym = install("N.groups");
+  setAttrib(ans_i, ngroups_sym, ScalarInteger(g));
   UNPROTECT(1);
   return ans_i;
 }
@@ -142,10 +198,11 @@ SEXP dupVecIndex(SEXP x) {
 
 SEXP dupVecIndexKeepNA(SEXP x) {
   const int n = length(x);
-  int K, tx = TYPEOF(x);
+  int K = 0, tx = TYPEOF(x);
   size_t M;
-  if(n >= INT_MAX) error("Length of 'x' is too large. (Long vector not supported yet)"); // 1073741824
+  // if(n >= INT_MAX) error("Length of 'x' is too large. (Long vector not supported yet)"); // 1073741824
   if (tx == STRSXP || tx == REALSXP || tx == CPLXSXP ) {
+    bigint:;
     const size_t n2 = 2U * (size_t) n;
     M = 256;
     K = 8;
@@ -154,23 +211,26 @@ SEXP dupVecIndexKeepNA(SEXP x) {
       K++;
     }
   } else if(tx == INTSXP) {
-    if(isFactor(x)) {
+    if(isFactor(x) || inherits(x, "qG")) {
       tx = 1000;
-      M = (size_t)nlevels(x) + 2;
-    } else M = (size_t)n;
+      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, install("N.groups"))) + 2;
+    } else {
+      int *p = INTEGER(x);
+      if(n > 10 && (NOGE(p[0], n) || NOGE(p[n/2], n) || NOGE(p[n-1], n))) goto bigint;
+      M = (size_t)n;
+    }
   } else if (tx == LGLSXP) {
     M = 3;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
-  // memset(h, 0, M * sizeof(int)); // not needed??
+  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
   SEXP ans_i = PROTECT(allocVector(INTSXP, n));
-  int *pans_i = INTEGER(ans_i);
-  size_t id = 0, g = 0;
+  int *restrict pans_i = INTEGER(ans_i), g = 0;
+  size_t id = 0;
   switch (tx) {
   case LGLSXP:
   case 1000: // This is for factors or logical vectors where the size of the table is known
   {
-    const int *px = INTEGER(x);
+    const int *restrict px = INTEGER(x);
     for (int i = 0, j; i != n; ++i) {
       if(px[i] == NA_INTEGER) {
         pans_i[i] = NA_INTEGER;
@@ -181,31 +241,51 @@ SEXP dupVecIndexKeepNA(SEXP x) {
       else pans_i[i] = h[j] = ++g;
     }
   } break;
-  case INTSXP: { // Faster version based on division hash...
-    const int *px = INTEGER(x);
-    unsigned int iid = 0, nu = (unsigned)n;
-    for (int i = 0; i != n; ++i) {
-      if(px[i] == NA_INTEGER) {
-        pans_i[i] = NA_INTEGER;
-        continue;
+  case INTSXP: {
+    const int *restrict px = INTEGER(x);
+    if(M == (size_t)n) { // Faster version based on division hash...
+      unsigned int iid = 0, nu = (unsigned)n;
+      for (int i = 0; i != n; ++i) {
+        if(px[i] == NA_INTEGER) {
+          pans_i[i] = NA_INTEGER;
+          continue;
+        }
+        iid = (unsigned)px[i];
+        if(iid >= nu) iid %= nu; // iid = (px[i] < n) ? px[i] : px[i] % n; // HASH(px[i], K); // get the hash value of x[i]
+        while(h[iid]) { // Check if this hash value has been seen before
+          if(px[h[iid]-1] == px[i]) { // Get the element of x that produced his value. if x[i] is the same, assign it the same index.
+            pans_i[i] = pans_i[h[iid]-1]; // h[id];
+            goto ibl;
+          } // else, we move forward to the next slot, until we find an empty one... We need to keep checking against the values,
+          // because if we found the same value before, we would also have put it in another slot after the initial one with the same hash value.
+          if(++id >= nu) id %= nu; // ++iid; iid %= nu; // # nocov
+        } // We put the index into the empty slot.
+        h[iid] = i + 1; // need + 1 because for zero the while loop gives false..
+        pans_i[i] = ++g; // h[id];
+        ibl:;
+      }
+    } else {
+      for (int i = 0; i != n; ++i) {
+        if(px[i] == NA_INTEGER) {
+          pans_i[i] = NA_INTEGER;
+          continue;
+        }
+        id = HASH(px[i], K);
+        while(h[id]) {
+          if(px[h[id]-1] == px[i]) {
+            pans_i[i] = pans_i[h[id]-1]; // h[id];
+            goto ibbl;
+          }
+          if(++id >= M) id %= M; // ++id; id %= M;
+        }
+        h[id] = i + 1;
+        pans_i[i] = ++g; // h[id];
+        ibbl:;
       }
-      iid = (unsigned)px[i];
-      if(iid >= nu) iid %= nu; // iid = (px[i] < n) ? px[i] : px[i] % n; // HASH(px[i], K); // get the hash value of x[i]
-      while(h[iid]) { // Check if this hash value has been seen before
-        if(px[h[iid]-1] == px[i]) { // Get the element of x that produced his value. if x[i] is the same, assign it the same index.
-          pans_i[i] = pans_i[h[iid]-1]; // h[id];
-          goto ibl;
-        } // else, we move forward to the next slot, until we find an empty one... We need to keep checking against the values,
-        // because if we found the same value before, we would also have put it in another slot after the initial one with the same hash value.
-        if(++id >= nu) id %= nu; // ++iid; iid %= nu; // # nocov
-      } // We put the index into the empty slot.
-      h[iid] = i + 1; // need + 1 because for zero the while loop gives false..
-      pans_i[i] = ++g; // h[id];
-      ibl:;
     }
   } break;
   case REALSXP: {
-    const double *px = REAL(x);
+    const double *restrict px = REAL(x);
     union uno tpv;
     for (int i = 0; i != n; ++i) {
       if(ISNAN(px[i])) {
@@ -227,13 +307,12 @@ SEXP dupVecIndexKeepNA(SEXP x) {
     }
   } break;
   case CPLXSXP: {
-    const Rcomplex *px = COMPLEX(x);
+    const Rcomplex *restrict px = COMPLEX(x);
     unsigned int u;
     union uno tpv;
     Rcomplex tmp;
     for (int i = 0; i != n; ++i) {
-      tmp.r = (px[i].r == 0.0) ? 0.0 : px[i].r;
-      tmp.i = (px[i].i == 0.0) ? 0.0 : px[i].i;
+      tmp = px[i];
       if(C_IsNA(tmp) || C_IsNaN(tmp)) {
         pans_i[i] = NA_INTEGER;
         continue;
@@ -256,7 +335,7 @@ SEXP dupVecIndexKeepNA(SEXP x) {
     }
   } break;
   case STRSXP: {
-    const SEXP *px = STRING_PTR(x);
+    const SEXP *restrict px = STRING_PTR(x);
     for (int i = 0; i != n; ++i) {
       if(px[i] == NA_STRING) {
         pans_i[i] = NA_INTEGER;
@@ -277,29 +356,33 @@ SEXP dupVecIndexKeepNA(SEXP x) {
   } break;
   }
   Free(h);
-  setAttrib(ans_i, install("N.groups"), ScalarInteger(g));
+  SEXP ngroups_sym = install("N.groups");
+  setAttrib(ans_i, ngroups_sym, ScalarInteger(g));
   UNPROTECT(1);
   return ans_i;
 }
 
-// TODO: Only one M calculation
+// TODO: Only one M calculation ?
 // Think: If in the second grouping variable all entries are the same, you loop through the whole table for each value..
-// TODO: Speed up for real values, i.e. system.time(group(DHSBR[1:2])) and system.time(group(wlddev)) (date), especially repeated real values appear slow !!
-// --> But also integers is slow, i.e. system.time(group(DHSBR[1:2])) when DHSBR[2] is integer.
 
 // **************************************************
 // This function adds a second vector to the grouping
 // **************************************************
-int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
+int dupVecSecond(int *restrict pidx, int *restrict pans_i, SEXP x, const int n, const int ng) {
 
   if(length(x) != n) error("Unequal length columns");
-  int K, tx = TYPEOF(x);
+  int K = 0, tx = TYPEOF(x), anyNA = 1;
   size_t M;
   if (tx == INTSXP || tx == STRSXP || tx == REALSXP || tx == CPLXSXP ) {
-    if(tx == INTSXP && isFactor(x) && (nlevels(x)+1) * ng <= 3 * n) {
-      tx = 1000;
-      M = (size_t)(nlevels(x)+1) * ng + 1;
-    } else {
+    if(tx == INTSXP && (isFactor(x) || inherits(x, "qG"))) {
+      K = isFactor(x) ? nlevels(x)+1 : asInteger(getAttrib(x, install("N.groups")))+1;
+      anyNA = !inherits(x, "na.included");
+      if(K * ng <= 3 * n) {
+        tx = 1000;
+        M = (size_t)(K * ng + 1);
+      } else K = 0;
+    }
+    if(K == 0) {
       const size_t n2 = 2U * (size_t) n;
       M = 256;
       K = 8;
@@ -312,25 +395,33 @@ int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
   } else if (tx == LGLSXP) {
     M = (size_t)ng * 3 + 1;
   } else error("Type %s is not supported.", type2char(tx)); // # nocov
-  int *h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
-  size_t id = 0, g = 0, hid = 0;
+  int *restrict h = (int*)Calloc(M, int), g = 0, hid = 0; // Table to save the hash values, table has size M
+  size_t id = 0;
   switch (tx) {
   case LGLSXP:
   {
-    const int *px = LOGICAL(x);
-    for (int i = 0, j; i != n; ++i) {
-      j = (px[i] == NA_LOGICAL) ? pidx[i] : pidx[i] + (px[i] + 1) * ng;
-      if(h[j]) pans_i[i] = h[j];
-      else pans_i[i] = h[j] = ++g;
+    const int *restrict px = LOGICAL(x);
+    for (int i = 0; i != n; ++i) {
+      id = (px[i] == NA_LOGICAL) ? pidx[i] : pidx[i] + (px[i] + 1) * ng;
+      if(h[id]) pans_i[i] = h[id];
+      else pans_i[i] = h[id] = ++g;
     }
   } break;
   case 1000: // This is for factors if feasible...
   {
-    const int *px = INTEGER(x);
-    for (int i = 0, j; i != n; ++i) {
-      j = (px[i] == NA_INTEGER) ? pidx[i] : pidx[i] + px[i] * ng;
-      if(h[j]) pans_i[i] = h[j];
-      else pans_i[i] = h[j] = ++g;
+    const int *restrict px = INTEGER(x);
+    if(anyNA) {
+      for (int i = 0; i != n; ++i) {
+        id = (px[i] == NA_INTEGER) ? pidx[i] : pidx[i] + px[i] * ng;
+        if(h[id]) pans_i[i] = h[id];
+        else pans_i[i] = h[id] = ++g;
+      }
+    } else {
+      for (int i = 0; i != n; ++i) {
+        id = pidx[i] + px[i] * ng;
+        if(h[id]) pans_i[i] = h[id];
+        else pans_i[i] = h[id] = ++g;
+      }
     }
   } break;
   // TODO: Think further about this! Perhaps you can also do this totally differently with a second vector capturing the unique values of idx!
@@ -338,7 +429,7 @@ int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
 
   // Note: In general, combining bitwise i.e. px[i] ^ pidx[i] seems slightly faster than multiplying (px[i] * pidx[i])...
   case INTSXP: {
-    const int *px = INTEGER(x);
+    const int *restrict px = INTEGER(x);
     for (int i = 0; i != n; ++i) {
       id = HASH(px[i] * pidx[i], K) + pidx[i]; // Need multiplication here instead of bitwise, see your benchmark with 100 mio. obs where second group is just sample.int(1e4, 1e8, T), there bitwise is very slow!!
       while(h[id]) {
@@ -355,7 +446,7 @@ int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
     }
   } break;
   case REALSXP: {
-    const double *px = REAL(x);
+    const double *restrict px = REAL(x);
     union uno tpv;
     for (int i = 0; i != n; ++i) {
       tpv.d = px[i]; // R_IsNA(px[i]) ? NA_REAL : (R_IsNaN(px[i]) ? R_NaN :px[i]);
@@ -374,14 +465,13 @@ int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
     }
   } break;
   case CPLXSXP: {
-    const Rcomplex *px = COMPLEX(x);
+    const Rcomplex *restrict px = COMPLEX(x);
     unsigned int u;
     union uno tpv;
     Rcomplex tmp;
     for (int i = 0; i != n; ++i) {
-      tmp.r = (px[i].r == 0.0) ? 0.0 : px[i].r;
-      tmp.i = (px[i].i == 0.0) ? 0.0 : px[i].i;
-      if (C_IsNA(tmp)) {
+      tmp = px[i];
+      if(C_IsNA(tmp)) {
         tmp.r = tmp.i = NA_REAL;
       } else if (C_IsNaN(tmp)) {
         tmp.r = tmp.i = R_NaN;
@@ -405,7 +495,7 @@ int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
     }
   } break;
   case STRSXP: {
-    const SEXP *px = STRING_PTR(x);
+    const SEXP *restrict px = STRING_PTR(x);
     for (int i = 0; i != n; ++i) {
       id = HASH(((intptr_t) px[i] & 0xffffffff) ^ pidx[i], K) + pidx[i];
       while(h[id]) {
@@ -423,9 +513,6 @@ int dupVecSecond(int *pidx, int *pans_i, SEXP x, const int n, const int ng) {
   } break;
   }
   Free(h);
-  // setAttrib(ans_i, sym_ng, ScalarInteger(g));
-  // UNPROTECT(1);
-  // return ans_i;
   return g;
 }
 
@@ -441,7 +528,7 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
   SEXP idx = islist ? dupVecIndex(VECTOR_ELT(X, 0)) : dupVecIndex(X);
   if(!(islist && l > 1) && start == 0 && size == 0) return idx; // l == 1 &&
   PROTECT(idx); ++nprotect;
-  SEXP sym_ng = PROTECT(install("N.groups")), res; ++nprotect;
+  SEXP sym_ng = install("N.groups"), res;
   int ng = asInteger(getAttrib(idx, sym_ng)), n = length(idx);
   if(islist && l > 1) {
     SEXP ans = PROTECT(allocVector(INTSXP, n)); ++nprotect;
@@ -462,9 +549,9 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
     PROTECT(res); ++nprotect;
     int *pres = INTEGER(res);
     if(start && size) { // Protect res ??
-      SEXP gs, st;
-      setAttrib(res, install("starts"), st = allocVector(INTSXP, ng));
-      setAttrib(res, install("group.sizes"), gs = allocVector(INTSXP, ng));
+      SEXP gs, st, starts_sym = install("starts"), sizes_sym = install("group.sizes");
+      setAttrib(res, starts_sym, st = allocVector(INTSXP, ng));
+      setAttrib(res, sizes_sym, gs = allocVector(INTSXP, ng));
       int *pgs = INTEGER(gs), *pst = INTEGER(st);
       memset(pgs, 0, sizeof(int) * ng); --pgs;
       memset(pst, 0, sizeof(int) * ng); --pst;
@@ -473,8 +560,8 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
         if(pst[pres[i]] == 0) pst[pres[i]] = i + 1;
       }
     } else if(start) {
-      SEXP st;
-      setAttrib(res, install("starts"), st = allocVector(INTSXP, ng));
+      SEXP st, starts_sym = install("starts");
+      setAttrib(res, starts_sym, st = allocVector(INTSXP, ng));
       int *pst = INTEGER(st), k = 0;
       memset(pst, 0, sizeof(int) * ng); --pst;
       for(int i = 0; i != n; ++i) {
@@ -484,8 +571,8 @@ SEXP groupVec(SEXP X, SEXP starts, SEXP sizes) {
         }
       }
     } else {
-      SEXP gs;
-      setAttrib(res, install("group.sizes"), gs = allocVector(INTSXP, ng));
+      SEXP gs, sizes_sym = install("group.sizes");
+      setAttrib(res, sizes_sym, gs = allocVector(INTSXP, ng));
       int *pgs = INTEGER(gs);
       memset(pgs, 0, sizeof(int) * ng); --pgs;
       for(int i = 0; i != n; ++i) ++pgs[pres[i]];
@@ -504,11 +591,9 @@ SEXP groupAtVec(SEXP X, SEXP starts, SEXP naincl) {
   SEXP idx = nain ? dupVecIndex(X) : dupVecIndexKeepNA(X);
   if(start == 0) return idx;
   PROTECT(idx);
-  SEXP sym_ng = PROTECT(install("N.groups"));
-  int ng = asInteger(getAttrib(idx, sym_ng)), n = length(idx);
-  int *pidx = INTEGER(idx);
-  SEXP st;
-  setAttrib(idx, install("starts"), st = allocVector(INTSXP, ng));
+  SEXP st, sym_ng = install("N.groups"), starts_sym = install("starts");
+  int ng = asInteger(getAttrib(idx, sym_ng)), n = length(idx), *pidx = INTEGER(idx);
+  setAttrib(idx, starts_sym, st = allocVector(INTSXP, ng));
   int *pst = INTEGER(st), k = 0;
   memset(pst, 0, sizeof(int) * ng); --pst;
   if(nain) {
@@ -526,11 +611,180 @@ SEXP groupAtVec(SEXP X, SEXP starts, SEXP naincl) {
       }
     }
   }
-  UNPROTECT(2);
+  UNPROTECT(1);
   return idx;
 }
 
 
+// Same as dupVecIndex, but saves group starts and returns unique values
+SEXP funiqueC(SEXP x) {
+  const int n = length(x);
+  if(n <= 1) return x;
+  int K = 0, tx = TYPEOF(x);
+  size_t M;
+  // if(n >= INT_MAX) error("Length of 'x' is too large. (Long vector not supported yet)"); // 1073741824
+  if (tx == STRSXP || tx == REALSXP || tx == CPLXSXP) {
+    bigint:;
+    const size_t n2 = 2U * (size_t) n;
+    M = 256;
+    K = 8;
+    while (M < n2) {
+      M *= 2;
+      K++;
+    }
+  } else if(tx == INTSXP) {
+    if(isFactor(x) || inherits(x, "qG")) {
+      tx = 1000;
+      M = isFactor(x) ? (size_t)nlevels(x) + 2 : (size_t)asInteger(getAttrib(x, install("N.groups"))) + 2;
+    } else {
+      int *p = INTEGER(x);
+      if(n > 10 && (NOGE(p[0], n) || NOGE(p[n/2], n) || NOGE(p[n-1], n))) goto bigint;
+      M = (size_t)n;
+    }
+  } else if (tx == LGLSXP) {
+    M = 3;
+  } else error("Type %s is not supported.", type2char(tx)); // # nocov
+  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values, table has size M
+  int *restrict st = (int*)R_alloc((tx == LGLSXP || tx == 1000) ? (int)M : n, sizeof(int));
+  int g = 0;
+  size_t id = 0;
+  SEXP res = R_NilValue;
+  switch (tx) {
+  case LGLSXP:
+  case 1000: // This is for factors or logical vectors where the size of the table is known
+  {
+    const int *restrict px = INTEGER(x);
+    if(tx == 1000 && inherits(x, "na.included")) {
+      for(int i = 0, k = (int)M-1, ng = k-1; i != n; ++i) {
+        if(h[px[i]]) continue;
+        h[px[i]] = 1;
+        st[g] = i;
+        if(++g == ng) break;
+      }
+    } else {
+      int  ng = tx == LGLSXP ? 3 : (int)M-1;
+      for(int i = 0, j, k = (int)M-1; i != n; ++i) {
+        j = (px[i] == NA_INTEGER) ? k : px[i];
+        if(h[j]) continue;
+        h[j] = 1;
+        st[g] = i;
+        if(++g == ng) break;
+      }
+    }
+    Free(h);
+    if(g == n) return x;
+    PROTECT(res = allocVector(tx == LGLSXP ? LGLSXP : INTSXP, g));
+    int *restrict pres = INTEGER(res);
+    for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
+  } break;
+  case INTSXP: {
+    const int *restrict px = INTEGER(x);
+    if(M == (size_t)n) { // Faster version based on division hash...
+      unsigned int iid = 0, nu = (unsigned)n;
+      for (int i = 0; i != n; ++i) {
+        iid = (unsigned)px[i];
+        if(iid >= nu) iid %= nu;
+        while(h[iid]) {
+          if(px[h[iid]-1] == px[i]) goto ibl;
+          if(++iid >= nu) iid %= nu;
+        }
+        h[iid] = i + 1;
+        st[g++] = i;
+        ibl:;
+      }
+    } else {
+      for (int i = 0; i != n; ++i) {
+        id = HASH(px[i], K);
+        while(h[id]) {
+          if(px[h[id]-1] == px[i]) goto ibbl;
+          if(++id >= M) id %= M;
+        }
+        h[id] = i + 1;
+        st[g++] = i;
+        ibbl:;
+      }
+    }
+    Free(h);
+    if(g == n) return x;
+    PROTECT(res = allocVector(INTSXP, g));
+    int *restrict pres = INTEGER(res);
+    for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
+  } break;
+  case REALSXP: {
+    const double *restrict px = REAL(x);
+    union uno tpv;
+    for (int i = 0; i != n; ++i) {
+      tpv.d = px[i];
+      id = HASH(tpv.u[0] + tpv.u[1], K);
+      while(h[id]) {
+        if(REQUAL(px[h[id]-1], px[i])) goto rbl;
+        if(++id >= M) id %= M;
+      }
+      h[id] = i + 1;
+      st[g++] = i;
+      rbl:;
+    }
+    Free(h);
+    if(g == n) return x;
+    PROTECT(res = allocVector(REALSXP, g));
+    double *restrict pres = REAL(res);
+    for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
+  } break;
+  case CPLXSXP: {
+    const Rcomplex *restrict px = COMPLEX(x);
+    unsigned int u;
+    union uno tpv;
+    Rcomplex tmp;
+    for (int i = 0; i != n; ++i) {
+      tmp = px[i];
+      if(C_IsNA(tmp)) {
+        tmp.r = tmp.i = NA_REAL;
+      } else if (C_IsNaN(tmp)) {
+        tmp.r = tmp.i = R_NaN;
+      }
+      tpv.d = tmp.r;
+      u = tpv.u[0] ^ tpv.u[1];
+      tpv.d = tmp.i;
+      u ^= tpv.u[0] ^ tpv.u[1];
+      id = HASH(u, K);
+      while(h[id]) {
+        if(CEQUAL(px[h[id]-1], px[i])) goto cbl;
+        if(++id >= M) id %= M; // # nocov
+      }
+      h[id] = i + 1;
+      st[g++] = i;
+      cbl:;
+    }
+    Free(h);
+    if(g == n) return x;
+    PROTECT(res = allocVector(CPLXSXP, g));
+    Rcomplex *restrict pres = COMPLEX(res);
+    for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
+  } break;
+  case STRSXP: {
+    const SEXP *restrict px = STRING_PTR(x);
+    for (int i = 0; i != n; ++i) {
+      id = HASH(((intptr_t) px[i] & 0xffffffff), K);
+      while(h[id]) {
+        if(px[h[id]-1] == px[i]) goto sbl;
+        if(++id >= M) id %= M; // # nocov
+      }
+      h[id] = i + 1;
+      st[g++] = i;
+      sbl:;
+    }
+    Free(h);
+    if(g == n) return x;
+    PROTECT(res = allocVector(STRSXP, g));
+    SEXP *restrict pres = STRING_PTR(res);
+    for(int i = 0; i != g; ++i) pres[i] = px[st[i]];
+  } break;
+  }
+  copyMostAttrib(x, res);
+  UNPROTECT(1);
+  return res;
+}
+
 
 // From the kit package...
 
