@@ -104,10 +104,10 @@ test_that("GRP works as intended", {
  expect_equal(GRP(wlddev2$country)[[1]], attributes(qG(wlddev2$country))[[1]])
  expect_equal(GRP(wlddev2$PCGDP)[[1]], attributes(qG(wlddev2$PCGDP, na.exclude = FALSE))[[1]])
 
- expect_equal(GRP(mtcars$mpg)[[4]][[1]], attributes(qG(mtcars$mpg, return.groups = TRUE))[[2]])
- expect_equal(GRP(mtcars$cyl)[[4]][[1]], attributes(qG(mtcars$cyl, return.groups = TRUE))[[2]])
- expect_equal(GRP(wlddev2$country)[[4]][[1]], attributes(qG(wlddev2$country, return.groups = TRUE))[[2]])
- expect_equal(GRP(wlddev2$PCGDP)[[4]][[1]], attributes(qG(wlddev2$PCGDP, na.exclude = FALSE, return.groups = TRUE))[[2]])
+ expect_equal(GRP(mtcars$mpg)[[4]][[1]], attributes(qG(mtcars$mpg, return.groups = TRUE))[["groups"]])
+ expect_equal(GRP(mtcars$cyl)[[4]][[1]], attributes(qG(mtcars$cyl, return.groups = TRUE))[["groups"]])
+ expect_equal(GRP(wlddev2$country)[[4]][[1]], attributes(qG(wlddev2$country, return.groups = TRUE))[["groups"]])
+ expect_equal(GRP(wlddev2$PCGDP)[[4]][[1]], attributes(qG(wlddev2$PCGDP, na.exclude = FALSE, return.groups = TRUE))[["groups"]])
 
  expect_visible(GRP(1:10))
  expect_visible(GRP(1:10, decreasing = TRUE))
@@ -316,22 +316,39 @@ base_group <- function(x, sort = FALSE, group.sizes = FALSE) {
 }
 
 test_that("group() works as intended", {
-  wlduo <- wlddev[order(rnorm(nrow(wldNA))), ]
+  wlduo <- wlddev[order(rnorm(nrow(wlddev))), ]
+  wlduoNA <- na_insert(wlduo)
   dlist <- c(mtcNA, wlddev, wlduo, GGDCNA, airquality)
   # Single grouping variable
   expect_identical(lapply(dlist, group, group.sizes = TRUE), lapply(dlist, base_group, group.sizes = TRUE))
   # Multiple grouping variables
-  g <- replicate(50, sample.int(11, sample.int(6, 1)), simplify = FALSE)
+  g <- replicate(70, sample.int(11, sample.int(6, 1)), simplify = FALSE)
   expect_identical(lapply(g, function(i) group(.subset(mtcars, i), group.sizes = TRUE)), lapply(g, function(i) base_group(.subset(mtcars, i), group.sizes = TRUE)))
-  g <- replicate(30, sample.int(13, sample.int(4, 1)), simplify = FALSE)
+  expect_identical(lapply(g, function(i) group(.subset(mtcNA, i), group.sizes = TRUE)), lapply(g, function(i) base_group(.subset(mtcNA, i), group.sizes = TRUE)))
+  g <- replicate(50, sample.int(13, sample.int(4, 1)), simplify = FALSE)
   expect_identical(lapply(g, function(i) group(.subset(wlduo, i), group.sizes = TRUE)), lapply(g, function(i) base_group(.subset(wlduo, i), group.sizes = TRUE)))
-  g <- replicate(30, sample.int(13, 3, replace = TRUE), simplify = FALSE)
+  expect_identical(lapply(g, function(i) group(.subset(wlduoNA, i), group.sizes = TRUE)), lapply(g, function(i) base_group(.subset(wlduoNA, i), group.sizes = TRUE)))
+  g <- replicate(50, sample.int(13, 3, replace = TRUE), simplify = FALSE)
   expect_identical(lapply(g, function(i) group(.subset(wlduo, i), group.sizes = TRUE)), lapply(g, function(i) base_group(.subset(wlduo, i), group.sizes = TRUE)))
+  expect_identical(lapply(g, function(i) group(.subset(wlduoNA, i), group.sizes = TRUE)), lapply(g, function(i) base_group(.subset(wlduoNA, i), group.sizes = TRUE)))
   # Positive and negative values give the same grouping
   nwld <- nv(wlduo)
   expect_identical(lapply(nwld, group), lapply(nwld %c*% -1, group))
   expect_visible(group(nwld %c*% -1))
   expect_visible(group(nwld[c(4,2,3)] %c*% -1))
+
+  expect_equal(group(0), base_group(0))
+  expect_equal(group(1), base_group(1))
+  expect_equal(group(0L), base_group(0L))
+  expect_equal(group(1L), base_group(1L))
+  expect_equal(group(Inf), base_group(Inf))
+  expect_equal(group(-Inf), base_group(-Inf))
+  expect_equal(group(c(NaN, NA, 0, 1, Inf, -Inf)), base_group(c(NaN, NA, 0, 1, Inf, -Inf)))
+  expect_equal(group(NA_integer_), base_group(NA_integer_))
+  expect_equal(group(NA_real_), base_group(NA_real_))
+  expect_equal(group(NaN), base_group(NaN))
+  expect_equal(group(NA), base_group(NA))
+  expect_equal(group(NA_character_), base_group(NA_character_))
 })
 
 GRP2 <- function(x) {
@@ -409,6 +426,9 @@ test_that("finteraction works as intended", {
   expect_equal(`oldClass<-`(finteraction(fl), "factor"), base::interaction(fl, drop = TRUE, lex.order = TRUE))
   expect_equal(`oldClass<-`(finteraction(ss(fl, 1:300)), "factor"), base::interaction(ss(fl, 1:300), drop = TRUE, lex.order = TRUE)) # missing levels
 
+  expect_equal(unattrib(finteraction(fl, factor = FALSE, sort = TRUE)), unattrib(base::interaction(fl, drop = TRUE, lex.order = TRUE)))
+  expect_equal(unattrib(finteraction(fl, factor = FALSE, sort = FALSE)), unattrib(group(fl)))
+
   # Missing value behavior is always different !!
   # expect_equal(`oldClass<-`(finteraction(flNA), "factor"), factor(base::interaction(flNA, drop = TRUE, lex.order = TRUE), exclude = NULL))
   # expect_equal(`oldClass<-`(finteraction(ss(flNA, 1:300)), "factor"), base::interaction(ss(flNA, 1:300), drop = TRUE, lex.order = TRUE))
@@ -429,8 +449,8 @@ test_that("fdroplevels works as intended", {
   expect_identical(fdroplevels(wld150$iso3c), droplevels(wld150$iso3c))
   expect_identical(fdroplevels(wldNA150$iso3c), droplevels(wldNA150$iso3c))
   expect_message(fdroplevels(1:3))
-  expect_warning(fdroplevels(wld150, bla = 1))
-  expect_warning(fdroplevels(wld150$iso3c, bla = 1))
+  # expect_warning(fdroplevels(wld150, bla = 1))
+  # expect_warning(fdroplevels(wld150$iso3c, bla = 1))
   expect_error(fdroplevels.factor(wld150$country))
 
 })
@@ -463,4 +483,11 @@ test_that("funique works well", {
  expect_equal(funique(rdatNA), setRownames(unique(rdatNA)))
  expect_equal(funique(rdatNA, sort = TRUE), roworderv(unique(rdatNA)))
 
+ expect_equal(lapply(wlddev, function(x) unattrib(base::unique(x))),
+              lapply(wlddev, function(x) unattrib(funique(x))))
+ expect_equal(lapply(wldNA, function(x) unattrib(base::unique(x))),
+              lapply(wldNA, function(x) unattrib(funique(x))))
+ expect_equal(lapply(GGDC10S, function(x) unattrib(base::unique(x))),
+              lapply(GGDC10S, function(x) unattrib(funique(x))))
+
 })
