@@ -82,8 +82,6 @@ wBY <- function(x, f, FUN, w, ...) {
   qDF(dapply(x, function(xi) unlist(Map(FUN, split(xi, f), wspl, ...), use.names = FALSE), return = "matrix"))
 }
 
-rn <- function(x) setRownames(x, NULL)
-
 # fbetween
 
 test_that("fbetween performs like between", {
@@ -112,14 +110,14 @@ test_that("fbetween performs like between", {
   expect_equal(fbetween(x, f, na.rm = FALSE), BY(x, f, between, use.g.names = FALSE))
   expect_equal(fbetween(xNA, f, na.rm = FALSE), BY(xNA, f, between, use.g.names = FALSE))
   expect_equal(fbetween(xNA, f), BY(xNA, f, between, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fbetween(m, g)), BY(m, g, between, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fbetween(m, g, na.rm = FALSE)), BY(m, g, between, use.g.names = FALSE))
-  expect_equal(rn(fbetween(mNA, g, na.rm = FALSE)), BY(mNA, g, between, use.g.names = FALSE))
-  expect_equal(rn(fbetween(mNA, g)), BY(mNA, g, between, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fbetween(mtcars, g)), BY(mtcars, g, between, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fbetween(mtcars, g, na.rm = FALSE)), BY(mtcars, g, between, use.g.names = FALSE))
-  expect_equal(rn(fbetween(mtcNA, g, na.rm = FALSE)), BY(mtcNA, g, between, use.g.names = FALSE))
-  expect_equal(rn(fbetween(mtcNA, g)), BY(mtcNA, g, between, na.rm = TRUE, use.g.names = FALSE))
+  expect_equal(fbetween(m, g), BY(m, g, between, na.rm = TRUE, use.g.names = FALSE))
+  expect_equal(fbetween(m, g, na.rm = FALSE), BY(m, g, between, use.g.names = FALSE))
+  expect_equal(fbetween(mNA, g, na.rm = FALSE), BY(mNA, g, between, use.g.names = FALSE))
+  expect_equal(fbetween(mNA, g), BY(mNA, g, between, na.rm = TRUE, use.g.names = FALSE))
+  expect_equal(fbetween(mtcars, g), BY(mtcars, g, between, na.rm = TRUE, use.g.names = FALSE))
+  expect_equal(fbetween(mtcars, g, na.rm = FALSE), BY(mtcars, g, between, use.g.names = FALSE))
+  expect_equal(fbetween(mtcNA, g, na.rm = FALSE), BY(mtcNA, g, between, use.g.names = FALSE))
+  expect_equal(fbetween(mtcNA, g), BY(mtcNA, g, between, na.rm = TRUE, use.g.names = FALSE))
 })
 
 test_that("fbetween performs like fbetween with weights all equal", {
@@ -510,7 +508,7 @@ test_that("B.data.frame method is foolproof", {
   expect_error(B(wlddev, w = 4))
   expect_error(B(wlddev, w = "year"))
   expect_error(B(wlddev, w = ~year2))
-  suppressWarnings(expect_error(B(wlddev, w = ~year + region)))
+  # suppressWarnings(expect_error(B(wlddev, w = ~year + region)))
   expect_error(B(wlddev, mtcars))
   expect_error(B(wlddev, 2))
   expect_error(B(wlddev, "iso3c"))
@@ -558,18 +556,18 @@ test_that("fwithin performs like within", {
   expect_equal(fwithin(mtcars, na.rm = FALSE), dapply(mtcars, within))
   expect_equal(fwithin(mtcNA, na.rm = FALSE), dapply(mtcNA, within))
   expect_equal(fwithin(mtcNA), dapply(mtcNA, within, na.rm = TRUE))
-  expect_equal(fwithin(x, f), BY(x, f, within, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(fwithin(x, f, na.rm = FALSE), BY(x, f, within, use.g.names = FALSE))
-  expect_equal(fwithin(xNA, f, na.rm = FALSE), BY(xNA, f, within, use.g.names = FALSE))
-  expect_equal(fwithin(xNA, f), BY(xNA, f, within, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fwithin(m, g)), BY(m, g, within, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fwithin(m, g, na.rm = FALSE)), BY(m, g, within, use.g.names = FALSE))
-  expect_equal(rn(fwithin(mNA, g, na.rm = FALSE)), BY(mNA, g, within, use.g.names = FALSE))
-  expect_equal(rn(fwithin(mNA, g)), BY(mNA, g, within, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fwithin(mtcars, g)), BY(mtcars, g, within, na.rm = TRUE, use.g.names = FALSE))
-  expect_equal(rn(fwithin(mtcars, g, na.rm = FALSE)), BY(mtcars, g, within, use.g.names = FALSE))
-  expect_equal(rn(fwithin(mtcNA, g, na.rm = FALSE)), BY(mtcNA, g, within, use.g.names = FALSE))
-  expect_equal(rn(fwithin(mtcNA, g)), BY(mtcNA, g, within, na.rm = TRUE, use.g.names = FALSE))
+  expect_equal(fwithin(x, f), BY(x, f, within, na.rm = TRUE))
+  expect_equal(fwithin(x, f, na.rm = FALSE), BY(x, f, within))
+  expect_equal(fwithin(xNA, f, na.rm = FALSE), BY(xNA, f, within))
+  expect_equal(fwithin(xNA, f), BY(xNA, f, within, na.rm = TRUE))
+  expect_equal(fwithin(m, g), BY(m, g, within, na.rm = TRUE))
+  expect_equal(fwithin(m, g, na.rm = FALSE), BY(m, g, within))
+  expect_equal(fwithin(mNA, g, na.rm = FALSE), BY(mNA, g, within))
+  expect_equal(fwithin(mNA, g), BY(mNA, g, within, na.rm = TRUE))
+  expect_equal(fwithin(mtcars, g), BY(mtcars, g, within, na.rm = TRUE))
+  expect_equal(fwithin(mtcars, g, na.rm = FALSE), BY(mtcars, g, within))
+  expect_equal(fwithin(mtcNA, g, na.rm = FALSE), BY(mtcNA, g, within))
+  expect_equal(fwithin(mtcNA, g), BY(mtcNA, g, within, na.rm = TRUE))
 })
 
 test_that("fwithin with custom mean performs like within (defined above)", {
@@ -586,14 +584,14 @@ test_that("fwithin with custom mean performs like within (defined above)", {
   expect_equal(fwithin(x, f, na.rm = FALSE, mean = 4.8456), BY(x, f, within, use.g.names = FALSE, mean = 4.8456))
   expect_equal(fwithin(xNA, f, na.rm = FALSE, mean = 4.8456), BY(xNA, f, within, use.g.names = FALSE, mean = 4.8456))
   expect_equal(fwithin(xNA, f, mean = 4.8456), BY(xNA, f, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(m, g, mean = 4.8456)), BY(m, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(m, g, na.rm = FALSE, mean = 4.8456)), BY(m, g, within, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(mNA, g, na.rm = FALSE, mean = 4.8456)), BY(mNA, g, within, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(mNA, g, mean = 4.8456)), BY(mNA, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(mtcars, g, mean = 4.8456)), BY(mtcars, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(mtcars, g, na.rm = FALSE, mean = 4.8456)), BY(mtcars, g, within, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(mtcNA, g, na.rm = FALSE, mean = 4.8456)), BY(mtcNA, g, within, use.g.names = FALSE, mean = 4.8456))
-  expect_equal(rn(fwithin(mtcNA, g, mean = 4.8456)), BY(mtcNA, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(m, g, mean = 4.8456), BY(m, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(m, g, na.rm = FALSE, mean = 4.8456), BY(m, g, within, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(mNA, g, na.rm = FALSE, mean = 4.8456), BY(mNA, g, within, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(mNA, g, mean = 4.8456), BY(mNA, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(mtcars, g, mean = 4.8456), BY(mtcars, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(mtcars, g, na.rm = FALSE, mean = 4.8456), BY(mtcars, g, within, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(mtcNA, g, na.rm = FALSE, mean = 4.8456), BY(mtcNA, g, within, use.g.names = FALSE, mean = 4.8456))
+  expect_equal(fwithin(mtcNA, g, mean = 4.8456), BY(mtcNA, g, within, na.rm = TRUE, use.g.names = FALSE, mean = 4.8456))
 })
 
 test_that("Centering on overall mean performs as indended", {
@@ -601,14 +599,14 @@ test_that("Centering on overall mean performs as indended", {
   expect_equal(fwithin(x, f, na.rm = FALSE, mean = "overall.mean"), BY(x, f, within, use.g.names = FALSE) + ave(x))
   # expect_equal(fwithin(xNA, f, na.rm = FALSE, mean = "overall.mean"), BY(xNA, f, within, use.g.names = FALSE) + B(xNA)) # Not the same !!
   expect_equal(fwithin(xNA, f, mean = "overall.mean"), BY(xNA, f, within, na.rm = TRUE, use.g.names = FALSE) + B(xNA))
-  expect_equal(rn(fwithin(m, g, mean = "overall.mean")), BY(m, g, within, na.rm = TRUE, use.g.names = FALSE) + B(m))
-  expect_equal(rn(fwithin(m, g, na.rm = FALSE, mean = "overall.mean")), BY(m, g, within, use.g.names = FALSE) + B(m))
+  expect_equal(fwithin(m, g, mean = "overall.mean"), BY(m, g, within, na.rm = TRUE, use.g.names = FALSE) + B(m))
+  expect_equal(fwithin(m, g, na.rm = FALSE, mean = "overall.mean"), BY(m, g, within, use.g.names = FALSE) + B(m))
   # expect_equal(fwithin(mNA, g, na.rm = FALSE, mean = "overall.mean"), BY(mNA, g, within, use.g.names = FALSE) + B(mNA))
-  expect_equal(rn(fwithin(mNA, g, mean = "overall.mean")), BY(mNA, g, within, na.rm = TRUE, use.g.names = FALSE) + B(mNA))
-  expect_equal(rn(fwithin(mtcars, g, mean = "overall.mean")), BY(mtcars, g, within, na.rm = TRUE, use.g.names = FALSE) + B(mtcars))
-  expect_equal(rn(fwithin(mtcars, g, na.rm = FALSE, mean = "overall.mean")), BY(mtcars, g, within, use.g.names = FALSE) + B(mtcars))
+  expect_equal(fwithin(mNA, g, mean = "overall.mean"), BY(mNA, g, within, na.rm = TRUE, use.g.names = FALSE) + B(mNA))
+  expect_equal(fwithin(mtcars, g, mean = "overall.mean"), BY(mtcars, g, within, na.rm = TRUE, use.g.names = FALSE) + B(mtcars))
+  expect_equal(fwithin(mtcars, g, na.rm = FALSE, mean = "overall.mean"), BY(mtcars, g, within, use.g.names = FALSE) + B(mtcars))
   # expect_equal(fwithin(mtcNA, g, na.rm = FALSE, mean = "overall.mean"), BY(mtcNA, g, within, use.g.names = FALSE) + B(mtcNA))
-  expect_equal(rn(fwithin(mtcNA, g, mean = "overall.mean")), BY(mtcNA, g, within, na.rm = TRUE, use.g.names = FALSE)+ B(mtcNA))
+  expect_equal(fwithin(mtcNA, g, mean = "overall.mean"), BY(mtcNA, g, within, na.rm = TRUE, use.g.names = FALSE)+ B(mtcNA))
 })
 
 test_that("fwithin performs like fwithin with weights all equal", {
@@ -1075,7 +1073,7 @@ test_that("W.data.frame method is foolproof", {
   expect_error(W(wlddev, w = 4))
   expect_error(W(wlddev, w = "year"))
   expect_error(W(wlddev, w = ~year2))
-  suppressWarnings(expect_error(W(wlddev, w = ~year + region)))
+  # suppressWarnings(expect_error(W(wlddev, w = ~year + region)))
   expect_error(W(wlddev, mtcars))
   expect_error(W(wlddev, 2))
   expect_error(W(wlddev, "iso3c"))
